#!/bin/bash
SCRIPT=$0
SCRIPT_NAME=$(basename "${SCRIPT}")

LOG_FILE=/tmp/${SCRIPT_NAME}.log
exec > >(tee -a ${LOG_FILE} )
exec 2> >(tee -a ${LOG_FILE} >&2)
LOG_TIME=`date "+%Y-%m-%d %H:%M:%S"`

echo starting ${SCRIPT_NAME} at ${LOG_TIME} 

[ "`id -u -n`" = "oracle" ] || (echo "$0 must be run as oracle" && exit 1)

# Fail immediately on any error
set -e

#Common dir is one level above where this script lives, and allows us to find the other peer folders
export COMMON_DIR=$(dirname "${SCRIPT}")/..
export EXT_FILE_DIR=$COMMON_DIR/external-files
export FILES_TO_COPY_DIR=$COMMON_DIR/files-to-copy-oracle
export SCRIPTS_DIR=$COMMON_DIR/scripts

#copy .bash_profile and PPSDEVGFE.env in and source it, which will set oracle environment variables
echo "Copying in oracle profile files and sourcing them"
cp $FILES_TO_COPY_DIR/.bash_profile ~/.
chmod -R 644 ~/.bash_profile
cp $FILES_TO_COPY_DIR/PPSDEVGFE.env ~/.
chmod -R 644 ~/PPSDEVGFE.env
. .bash_profile

echo "Installing Oracle Software"	
/tmp/database/runInstaller -silent -waitforcompletion -responseFile $SCRIPTS_DIR/db_install.rsp

echo "End of ${SCRIPT_NAME}"