#!/bin/bash
SCRIPT=$0
SCRIPT_NAME=$(basename "${SCRIPT}")
LOG_FILE=/tmp/${SCRIPT_NAME}.log
exec > >(tee -a ${LOG_FILE} )
exec 2> >(tee -a ${LOG_FILE} >&2)
LOG_TIME=`date "+%Y-%m-%d %H:%M:%S"`

echo starting ${SCRIPT_NAME} at ${LOG_TIME}

[ "`id -u -n`" = "oracle" ] || (echo "$0 must be run as oracle" && exit 1)
 
#Common dir is one level above where this script lives, and allows us to find the other peer folders
export COMMON_DIR=$(dirname "${SCRIPT}")/..
export EXT_FILE_DIR=$COMMON_DIR/external-files
export SCRIPTS_DIR=$COMMON_DIR/scripts

export DUMP_FILE_NAME="expdp_PPSDEV05_FULL_20170213.dmp"
cp $EXT_FILE_DIR/dump-files/$DUMP_FILE_NAME /u01/dbs/oracle/admin/PPSDEVGFE/dpdump/

LOG_FILE="impdp_PPSDEVGFE_FROM_expdp_PPSDEV05_FULL_20170213.log"

#replace <PASSWORD_HERE> with the password from the SYSTEMPASSWORD property in dbca.rsp.  Wrap the password in single quotes
$ORACLE_HOME/bin/impdp SYSTEM/'<PASSWORD_HERE>'@PPSDEVGFE DIRECTORY=DATA_PUMP_DIR SCHEMAS=DATUP,FDB_DIF,PPSNEPL,SDS TABLE_EXISTS_ACTION=REPLACE CONTENT=ALL DUMPFILE=$DUMP_FILE_NAME LOGFILE=$LOG_FILE

#replace <PASSWORD_HERE> with the password from the SYSTEMPASSWORD property in dbca.rsp.  Wrap the password in single quotes
#import just the KAAJEE schema without data.  Once KAAJEE is removed, we can remove this step
$ORACLE_HOME/bin/impdp SYSTEM/'<PASSWORD_HERE>'@PPSDEVGFE DIRECTORY=DATA_PUMP_DIR SCHEMAS=KAAJEE TABLE_EXISTS_ACTION=REPLACE CONTENT=METADATA_ONLY DUMPFILE=$DUMP_FILE_NAME LOGFILE=${LOG_FILE}_KAAJEE

echo "Running scripts to catch up with changes made since the dump that was loaded..."
$SCRIPTS_DIR/run-catchup-scripts.sh

#change to the dir where all the SQL scripts are so we don't have to qualify scripts
cd $SCRIPTS_DIR

echo "Creating public synonyms script..."
echo exit | $ORACLE_HOME/bin/sqlplus / as sysdba @CREATE_PUBLIC_SYNONYMS
echo "Running public synonyms script..."
echo exit | $ORACLE_HOME/bin/sqlplus / as sysdba @create-synonyms-dynamic
echo "Removing public synonyms script:..."
rm create-synonyms-dynamic.sql

echo "Updating National Settings for local SFTP..."
echo exit | $ORACLE_HOME/bin/sqlplus / as sysdba @UPDATE_NATIONAL_SETTINGS_FOR_LOCALHOST_FILE

echo "Recompiling schema..."
echo exit | $ORACLE_HOME/bin/sqlplus / as sysdba @RECOMPILE_SCHEMA

echo "End of ${SCRIPT_NAME}"
