/**
 * 
 */
package gov.va.med.pharmacy.peps.common.vista;

import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

/**
 * Class to define the Vista File and Field Numbers for the Product File, 50.68.
 * @since 3.0
 */
public class VistaProductFile extends VistaFile {

  /** Vista File Number for the Product File. */
  public static final String FILE_NUMBER = "50.68";
   
  /* FIELDS */   
  /** VistA Field Number used to denote a new Product. */
  public static final String NEW_PRODUCT_FIELD_NUMBER = ".01";
  
  public static final String MASTERVUID_FIELD_NUMBER = "99.98";

  public static final String VUID_FIELD_NUMBER = "99.99";

  public static final String CHILD_FIELD_NUMBER01 = "99.991,.01";

  public static final String CHILD_FIELD_NUMBER02 = "99.991,.02";

  public static final String GENERIC_IEN_FIELD_NUMBER = ".05";

  public static final String DOSAGEFORM_FIELD_NUMBER = "1";

  public static final String STRENGTH_FIELD_NUMBER = "2";

  public static final String PRODUCTUNIT_FIELD_NUMBER = "3";

  public static final String NATIONALFORMULARY_FIELD_NUMBER = "4";

  public static final String VAPRINTNAME_FIELD_NUMBER = "5";

  public static final String VAPRODUCTIDENTIFIER_FIELD_NUMBER = "6";

  /** Vista field number for CMOP Indicator. */
  public static final String CMOPINDICATOR_FIELD_NUMBER = "7";  
  
  public static final String DISPENSEUNIT_FIELD_NUMBER = "8";

  public static final String GCNSEQUENCE_FIELD_NUMBER = "11";

  public static final String PRIMARYDRUGCLASS_FIELD_NUMBER = "15";

  public static final String SECONDARYDRUGCLASS_FIELD_NUMBER = "16,.01";

  public static final String NATFORMULARYIND_FIELD_NUMBER = "17";

  public static final String CHILD_ING_FIELD_NUMBER01 = "14,.01";

  public static final String CHILD_ING_FIELD_NUMBER1 = "14,1";

  public static final String CHILD_ING_FIELD_NUMBER2 = "14,2";

  public static final String CSFEDSCH_FIELD_NUMBER = "19";

  public static final String SOURCE_FIELD_NUMBER = "20";

  public static final String STATUS_FIELD_NUMBER = "21";

  public static final String OVERRIDEDF_FIELD_NUMBER = "31";

  public static final String EXCLUDEDRUGINT_FIELD_NUMBER = "23";

  public static final String DEFAULTPOSDOSAGE_FIELD_NUMBER = "40";

  public static final String POSSIBLEDOSAGE_FIELD_NUMBER = "41";

  public static final String PRODUCTPACK_FIELD_NUMBER = "42";

  /** Base of the Field Numbers for Copay Tier. */
  public static final String COPAY_TIER_FIELD_NUMBER_BASE = "45";

  /** Field Number for the Copay Tier Value.  */
  public static final String COPAY_TIER_FIELD_NUMBER = COPAY_TIER_FIELD_NUMBER_BASE + ",.01";

  /** Field Number for the Copay Tier Start Date.  */
  public static final String COPAY_TIER_START_DATE_FIELD_NUMBER = COPAY_TIER_FIELD_NUMBER_BASE + ",1";

  /** Field Number for the Copay Tier End Date.  */
  public static final String COPAY_TIER_END_DATE_FIELD_NUMBER = COPAY_TIER_FIELD_NUMBER_BASE + ",2";

  public static final String FDAMEDGUIDE_FIELD_NUMBER = "100";

  public static final String SERVICECODE_FIELD_NUMBER = "2000";

  /** New hazardous waste fields for the file update process.**/
  public static final String HAZTOHAN_FIELD_NUMBER = "101";

  public static final String HAZTODIS_FIELD_NUMBER = "102";

  public static final String PRIMARY_EPA_CODE_FIELD_NUMBER = "103";

  public static final String WASTE_SORT_CODE_FIELD_NUMBER = "104";

  public static final String DOT_SHIPPING_CODE_FIELD_NUMBER = "105";

  /** VistA Field Number for the Formulary Designator.  */
  public static final String FORMULARY_DESIGNATOR_FIELD_NUMBER = "109";
    
  /** VistA Field Number for the product text.  VistA Data Dictionary calls it "FORMULARY DESIGNATOR TEXT".  */
  public static final String PRODUCT_TEXT_FIELD_NUMBER = "110";
  
  /** Field Number for the product coding system  */
  public static final String PRODUCT_CODING_SYSTEM_NUMBER = "43,.01";
  
  /** Field Number for the product coding system code  */
  public static final String PRODUCT_CODING_SYSTEM_CODE_NUMBER = "43,.02,.01";
  
  /** Set of VistA Required Fields on a New Product. */
  public static final Set<String> VISTA_REQUIRED_FIELDS = Collections.unmodifiableSet(new LinkedHashSet<String>(Arrays
      .asList(new String[] {
          NEW_PRODUCT_FIELD_NUMBER,
          MASTERVUID_FIELD_NUMBER,
          VUID_FIELD_NUMBER,
          CHILD_FIELD_NUMBER01,
          CHILD_FIELD_NUMBER02,          
          CHILD_ING_FIELD_NUMBER01,
          CHILD_ING_FIELD_NUMBER1,
          CHILD_ING_FIELD_NUMBER2,
          PRIMARYDRUGCLASS_FIELD_NUMBER,
          DOSAGEFORM_FIELD_NUMBER,
          STRENGTH_FIELD_NUMBER,
          PRODUCTUNIT_FIELD_NUMBER,
          GENERIC_IEN_FIELD_NUMBER,
          OVERRIDEDF_FIELD_NUMBER
      })));  
  
  public static final Set<String> VISTA_OPTIONAL_FIELDS = Collections.unmodifiableSet(new LinkedHashSet<String>(Arrays
      .asList(new String[] {
          CMOPINDICATOR_FIELD_NUMBER, 
          COPAY_TIER_END_DATE_FIELD_NUMBER, 
          COPAY_TIER_FIELD_NUMBER,  
          COPAY_TIER_START_DATE_FIELD_NUMBER, 
          CSFEDSCH_FIELD_NUMBER, 
          DEFAULTPOSDOSAGE_FIELD_NUMBER, 
          DISPENSEUNIT_FIELD_NUMBER, 
          DOT_SHIPPING_CODE_FIELD_NUMBER, 
          EXCLUDEDRUGINT_FIELD_NUMBER, 
          FDAMEDGUIDE_FIELD_NUMBER, 
          FORMULARY_DESIGNATOR_FIELD_NUMBER, 
          GCNSEQUENCE_FIELD_NUMBER, 
          HAZTODIS_FIELD_NUMBER, 
          HAZTOHAN_FIELD_NUMBER, 
          NATFORMULARYIND_FIELD_NUMBER, 
          NATIONALFORMULARY_FIELD_NUMBER, 
          POSSIBLEDOSAGE_FIELD_NUMBER, 
          PRIMARY_EPA_CODE_FIELD_NUMBER, 
          PRODUCT_TEXT_FIELD_NUMBER, 
          PRODUCTPACK_FIELD_NUMBER, 
          SECONDARYDRUGCLASS_FIELD_NUMBER, 
          SERVICECODE_FIELD_NUMBER, 
          SOURCE_FIELD_NUMBER, 
          VAPRINTNAME_FIELD_NUMBER, 
          VAPRODUCTIDENTIFIER_FIELD_NUMBER, 
          WASTE_SORT_CODE_FIELD_NUMBER 
      })));  
}
