package gov.va.med.pharmacy.peps.common.vo;

import java.util.Collection;
import java.util.Set;

/**
 * Copay Tier Value Object.
 */
public class CopayTierVo extends ManagedDataVo {

    private static final long serialVersionUID = 1L;

    /**
     * String Variable Description
     */
    private String description; 
    
    /**
     * The constructor for Copay Tier.
     */
    public CopayTierVo() {

    }


    /**
     * Gets the Value from the FieldKey class.
     * @return value for Tier Level.
     */
    public final String getTierLevel() { 
        return getValue();
    }

    /**
     * Sets the Value from the FieldKey class.
     * @param tierLevel is returned as Value from the FieldKey class
     */
    public final void setTierLevel(String tierLevel) {
    
        setValue(tierLevel);
    }
  
    /**
     * Gets description of the copay tier.
     * @return description.
     */
    public final String getDescription() {
    
        return description;
    }
    
    public final void setDescription(String description) {
    
        this.description = description;
    }
    
    /**
     * Returns the domain group of the managed Data.
     * 
     * @return domainGroup
     */
    public final DomainGroup getDomainGroup() {
        return DomainGroup.COPAY_TIER;
    }
    
    /**
     * Returns true if this CopayTierVo is a local only domain.
     * 
     * @return false always
     */
    public final boolean isLocalOnlyData() {
        return false;
    }
    
    /**
     * Returns true if the domain is an NDF domain for the DrugClassVo.
     * 
     * @return boolean
     */
    public final boolean isNdf() {
        return true;
    }
    
    /**
     * Returns true if the domain is standardized for the CopayTierVo.
     * 
     * @return boolean
     */
    public final boolean isStandardized() {
        return false;
    }

    /**
     * This is the CopayTierVo method to return if this item requires two reviews 
     * before adds/modifies are approved or rejected  for the CopayTierVo.
     * <p>
     * Differs from {@link FieldKey#isRequiresSecondApproval()} as this method is used during adds and the FieldKey method is
     * used during mods. In addition, this method only applies to the CopayTierVo, not individual fields.
     * 
     * @return boolean True if this item requires two reviews, otherwise, only one review is needed.
     */
    @Override
    public final boolean isTwoReviewItem() {

        return false;
    } 
    
    
    /**
     * List all required fields for this ValueObject, with the pre-condition that the current instance is not a read-only
     * one.
     * 
     * @param environment the current {@link Environment}
     * @param roles Collection of {@link Role} instances for the current user
     * @return Set<FieldKey> All required fields for this CoayTierVo.
     */
    protected Set<FieldKey> handleListRequiredFields(Environment environment, Collection<Role> roles) {
        Set<FieldKey> requiredFields = super.handleListRequiredFields(environment, roles);

        requiredFields.add(FieldKey.DESCRIPTION);

        return requiredFields;
    }
    
    /**
     * Disable editing of description and item status fields if the user is a second approver, 
     * but is neither a manager nor a supervisor also.
     * 
     * Note: Viewers and migrators have all fields disabled by default in 
     * {@link UserCapabilityImpl#authorizeFields(gov.va.med.pharmacy.peps.common.vo.UserVo, 
     * java.lang.Object)}, so only the second approver role needs specific adjustment here.
     * 
     * @see gov.va.med.pharmacy.peps.common.vo.ManagedDataVo#handleListDisabledFields(
     * gov.va.med.pharmacy.peps.common.vo.Environment, java.util.Collection)
     */
    @Override
    protected Set<FieldKey> handleListDisabledFields(Environment environment, Collection<Role> roles) {
        Set<FieldKey> result = super.handleListDisabledFields(environment, roles);
        
        if(roles.contains(Role.PSS_PPSN_SECOND_APPROVER) 
            && !(roles.contains(Role.PSS_PPSN_MANAGER) || roles.contains(Role.PSS_PPSN_SUPERVISOR))) {
            result.add(FieldKey.DESCRIPTION);
            result.add(FieldKey.ITEM_STATUS);
        }
        
        return result;
    }    
    
}
