package gov.va.med.pharmacy.peps.common.vo;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

/**
 * Formulary Designator Value Object.
 */
public class FormularyDesignatorVo extends ManagedDataVo {

    private static final long serialVersionUID = 1L;

    /**
     * String Variable Description
     */
    private String description; 
    
    /**
     * The constructor for Formulary Designator.
     */
    public FormularyDesignatorVo() {

    }


    /**
     * Gets the Value from the FieldKey class.
     * @return value for formulary designator name.
     */
    public final String getName() { 
        return getValue();
    }

    /**
     * Sets the Value from the FieldKey class.
     * @param name is returned as Value from the FieldKey class
     */
    public final void setName(String name) {
        setValue(name);
    }
  
    /**
     * Gets description of the formulary designator.
     * @return description.
     */
    public final String getDescription() {
        return description;
    }
    
    public final void setDescription(String description) {
        this.description = description;
    }
    
    /**
     * Returns the domain group of the managed Data.
     * 
     * @return domainGroup
     */
    public final DomainGroup getDomainGroup() {
        return DomainGroup.GROUP_3;
    }
    
    /**
     * Returns true if this FormularyDesignatorVo is a local only domain.
     * 
     * @return false always
     */
    public final boolean isLocalOnlyData() {
        return false;
    }
    
    /**
     * Returns true if the domain is an NDF domain for the FormularyDesignatorVo.
     * 
     * @return boolean
     */
    public final boolean isNdf() {
        return true;
    }
    
    /**
     * Returns true if the domain is standardized for the FormularyDesignatorVo.
     * 
     * @return boolean
     */
    public final boolean isStandardized() {
        return false;
    }

    /**
     * This is the FormularyDesignatorVo method to return if this item requires two reviews 
     * before adds/modifies are approved or rejected  for the FormularyDesignatorVo.
     * <p>
     * Differs from {@link FieldKey#isRequiresSecondApproval()} as this method is used during adds and the FieldKey method is
     * used during mods. In addition, this method only applies to the FormularyDesignatorVo, not individual fields.
     * 
     * @return boolean True if this item requires two reviews, otherwise, only one review is needed.
     */
    @Override
    public final boolean isTwoReviewItem() {

        return false;
    } 
    
    /**
     * Disable editing of description and item status fields if the user is a second approver, 
     * but is neither a manager nor a supervisor also.
     * 
     * Note: Viewers and migrators have all fields disabled by default in 
     * {@link UserCapabilityImpl#authorizeFields(gov.va.med.pharmacy.peps.common.vo.UserVo, 
     * java.lang.Object)}, so only the second approver role needs specific adjustment here.
     * 
     * @see gov.va.med.pharmacy.peps.common.vo.ManagedDataVo#handleListDisabledFields(
     * gov.va.med.pharmacy.peps.common.vo.Environment, java.util.Collection)
     */
    @Override
    protected Set<FieldKey> handleListDisabledFields(Environment environment, Collection<Role> roles) {
        Set<FieldKey> result = super.handleListDisabledFields(environment, roles);
        
        if(roles.contains(Role.PSS_PPSN_SECOND_APPROVER) 
            && !(roles.contains(Role.PSS_PPSN_MANAGER) || roles.contains(Role.PSS_PPSN_SUPERVISOR))) {
            result.add(FieldKey.DESCRIPTION);
            result.add(FieldKey.ITEM_STATUS);
        }
        
        return result;
    }
    
    /**
     * List all required fields for this ValueObject, with the pre-condition that the current instance is not a read-only
     * one.
     * 
     * @param environment the current {@link Environment}
     * @param roles Collection of {@link Role} instances for the current user
     * @return Set<FieldKey> All required fields for this FormularyDesignatorVo.
     */
    protected Set<FieldKey> handleListRequiredFields(Environment environment, Collection<Role> roles) {
        Set<FieldKey> requiredFields = super.handleListRequiredFields(environment, roles);

        requiredFields.add(FieldKey.NAME);
        requiredFields.add(FieldKey.DESCRIPTION);

        return requiredFields;
    }
    
    /**
     * Returns a list of all fields appropriate for display on the wizard, sorted
     * descending so that the Formulary Designator field (a.k.a. "Name" and/or 
     * "Value" field) appears before the Description field.
     * 
     * @return collection of FieldKeys corresponding to form fields for editing
     * a formulary designator data element, in descending order by key.
     */
    @Override
    public Collection<FieldKey> listAllWizardFields() {
        List<FieldKey> fields = new ArrayList<FieldKey>(super.listAllWizardFields());
        Comparator<FieldKey> comparator = 
            (FieldKey first, FieldKey second) 
                -> second.getKey().toString().compareTo(first.getKey().toString());
        Collections.sort(fields, comparator);
        return fields;
    }
    
}
