package gov.va.med.pharmacy.peps.common.vo;

import java.util.Date;

/**
 * This class represents a single row of data for the 'Products with Changes
 * to Copay Tier Data' report.
 */
public class ReportCopayTierChangeVo {
    
    /**
     * Unique identifier for the product.
     */
    private Long eplId;
    
    /**
     * The VA Product Name.
     */
    private String vaProductName;
    
    /**
     * The CMOP unique identifier for the product.
     */
    private String cmopId;
    
    /**
     * The national formulary name.
     */
    private String nationalFormularyName;
    
    /**
     * For inactive products, the date the product was
     * marked inactive.
     */
    private Date statusDate;
    
    /**
     * The current copay tier associated with the product.
     */
    private String tier;
    
    /**
     * The effective date for the product's current copay tier.
     */
    private Date effectiveDate;
    
    /**
     * The end date for the product's current copay tier.
     */
    private Date endDate;
    
    /**
     * The future copay tier associated with the product, if
     * defined.
     */
    private String futureTierLevel;
    
    /**
     * The effective date for the future copay tier associated with
     * the product, if defined.
     */
    private Date futureEffectiveDate;
    
    /**
     * A value comprised of all the product's associated drug class
     * codes, separated by a delimiter.
     */
    private String drugClass;
    
    /**
     * Get EPL_ID.
     * 
     * @return EPL_ID
     */
    public final Long getEplId() {    
        return eplId;
    }
    
    /**
     * Set EPL_ID.
     * 
     * @param eplId eplId
     */
    public final void setEplId(Long eplId) {    
        this.eplId = eplId;
    }
    
    /**
     * Get the VA product name.
     * 
     * @return VA product name
     */
    public final String getVaProductName() {    
        return vaProductName;
    }
    
    /**
     * Set the VA product name.
     * 
     * @param vaProductName vaProductName
     */
    public final void setVaProductName(String vaProductName) {    
        this.vaProductName = vaProductName;
    }
    
    /**
     * Get the CMOP_ID.
     * 
     * @return cmopId
     */
    public final String getCmopId() {    
        return cmopId;
    }
    
    /**
     * Set the CMOP_ID.
     * 
     * @param cmopId cmopId
     */
    public final void setCmopId(String cmopId) {    
        this.cmopId = cmopId;
    }
    
    /**
     * Get the national formulary name.
     * 
     * @return nationalFormularyName
     */
    public final String getNationalFormularyName() {    
        return nationalFormularyName;
    }
    
    /**
     * Set the national formulary name.
     * 
     * @param nationalFormularyName nationalFormularyName
     */
    public final void setNationalFormularyName(String nationalFormularyName) {    
        this.nationalFormularyName = nationalFormularyName;
    }
    
    /**
     * Get the status date.
     * 
     * @return statusDate
     */
    public final Date getStatusDate() {    
        return statusDate;
    }
    
    /**
     * Set the status date.
     * 
     * @param statusDate statusDate
     */
    public final void setStatusDate(Date statusDate) {    
        this.statusDate = statusDate;
    }
    
    /**
     * Get the current copay tier.
     * 
     * @return tier
     */
    public final String getTier() {    
        return tier;
    }
    
    /**
     * Set the current copay tier.
     * 
     * @param tier tier
     */
    public final void setTier(String tier) {    
        this.tier = tier;
    }
    
    /**
     * Get the current copay tier's effective date.
     * 
     * @return effectiveDate
     */
    public final Date getEffectiveDate() {    
        return effectiveDate;
    }
    
    /**
     * Set the current copay tier's effective date.
     * 
     * @param effectiveDate effectiveDate
     */
    public final void setEffectiveDate(Date effectiveDate) {    
        this.effectiveDate = effectiveDate;
    }
    
    /**
     * Get the current copay tier's end date.
     * 
     * @return endDate
     */
    public final Date getEndDate() {    
        return endDate;
    }
    
    /**
     * Set the current copay tier's end date.
     * 
     * @param endDate endDate
     */
    public final void setEndDate(Date endDate) {    
        this.endDate = endDate;
    }
    
    /**
     * Get the future copay tier.
     * 
     * @return futureTierLevel
     */
    public final String getFutureTierLevel() {    
        return futureTierLevel;
    }
    
    /**
     * Set the future copay tier.
     * 
     * @param futureTierLevel futureTierLevel
     */
    public final void setFutureTierLevel(String futureTierLevel) {    
        this.futureTierLevel = futureTierLevel;
    }
    
    /**
     * Get the future copay tier's effective date.
     * 
     * @return futureEffectiveDate
     */
    public final Date getFutureEffectiveDate() {    
        return futureEffectiveDate;
    }
    
    /**
     * Set the future copay tier's effective date.
     * 
     * @param futureEffectiveDate futureEffectiveDate
     */
    public final void setFutureEffectiveDate(Date futureEffectiveDate) {    
        this.futureEffectiveDate = futureEffectiveDate;
    }
    
    /**
     * Get the product's drug class(es).
     * 
     * @return drugClass
     */
    public final String getDrugClass() {    
        return drugClass;
    }
    
    /**
     * Set the product's drug class(es).
     * 
     * @param drugClass drugClass
     */
    public final void setDrugClass(String drugClass) {
        this.drugClass = drugClass;
    }    
}
