package gov.va.med.pharmacy.peps.common.test;

import java.util.Collection;

import org.easymock.EasyMock;
import org.easymock.IArgumentMatcher;

/**
 * EasyMock IArgumentMatcher to compare the expected value of a Collection parameter.
 */
public class CollectionMatcher implements IArgumentMatcher {

  /** The collection. */
  private Collection<?> collection;
  
  /** The not found. */
  private String notFound;

  /**
   * Instantiates a new collection matcher.
   *
   * @param collection the collection
   */
  public CollectionMatcher(Collection<?> collection) {
    this.collection = collection;
    notFound = "";
  }

  /**
   * Collection eq.  The main method to call when using this matcher.
   *
   * @param collection the collection
   * @return the collection
   */
  public static Collection<?> collectionEq(Collection<?> collection) {
    EasyMock.reportMatcher(new CollectionMatcher(collection));
    return null;
  }

  /* (non-Javadoc)
   * @see org.easymock.IArgumentMatcher#appendTo(java.lang.StringBuffer)
   */
  @Override
  public void appendTo(StringBuffer buffer) {
    buffer.append(notFound).append(" not found in {");

    String comma = "";
    for (Object o : collection) {
      buffer.append(comma).append(o.toString());
      comma = ",";
    }

    buffer.append("}");
  }

  /* (non-Javadoc)
   * @see org.easymock.IArgumentMatcher#matches(java.lang.Object)
   */
  @Override
  public boolean matches(Object otherCollection) {
    if (!(otherCollection instanceof Collection)) {
      return false;
    }

    Collection<?> otherText = (Collection<?>) otherCollection;

    for (Object o : otherText) {
      if (!collection.contains(o)) {
        notFound = o.toString();
        return false;
      }
    }

    return true;

  }

}
