package gov.va.med.pharmacy.peps.common.utility;

import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

import java.time.Instant;
import java.time.LocalDate;
import java.util.TimeZone;

import org.junit.Test;

/**
 * Unit tests for {@link DateTimeUtility}.
 */
public class DateTimeUtilityTest {

  /**
   * Tests {@link DateTimeUtility#toInstant(LocalDate)} returns the expected Instant.
   */
  @Test
  public void testToInstant() {
    final LocalDate localDate = LocalDate.of(2001, 2, 4);
    final long expectedSeconds = 981266400L;  // Midnight 2/4/2001 in GMT-6.
    
    /* Behavior & Verification */
    
    // Set the TimeZone to a known zone so that we know the exact expected epoch second.
    final TimeZone previousTimeZone = TimeZone.getDefault();
    TimeZone.setDefault(TimeZone.getTimeZone("GMT-6"));
    try {
      final Instant instant = DateTimeUtility.toInstant(localDate);
      assertThat(instant.getEpochSecond(), equalTo(expectedSeconds));
    } finally {
      // Make sure we set the TimeZone back.
      TimeZone.setDefault(previousTimeZone);
    }
  }

}
