package gov.va.med.pharmacy.peps.common.vo;

import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.test.generator.ProductGenerator;

/**
 * Automated tests for {@link RematchVo} .
 */
public class RematchVoTest {

  final private ProductGenerator productGenerator = new ProductGenerator();
  
  @Test
  public void testAutomaticRematch() {
    /* Setup */
    List<ProductVo> productList = productGenerator.getList();
    ProductVo fromProduct = productList.get(0);
    ProductVo toAutomaticProduct = productList.get(1);
    RematchVo rematch = RematchVo.automaticRematch(fromProduct, toAutomaticProduct, MatchReason.INACTIVATION);
    /* Behavior & Verification */
    assertEquals(true, rematch.getIsAutomaticRematch());
    assertEquals(toAutomaticProduct, rematch.getToAutomaticProduct());
    assertEquals(new HashSet<>(Arrays.asList(toAutomaticProduct)), rematch.getMatchProducts());

  }
  
  @Test
  public void testSuggestedRematch() {
    /* Setup */
    List<ProductVo> productList = productGenerator.getList();
    Set<ProductVo> suggestedProductSet = new HashSet<>(Arrays.asList(productList.get(1), productList.get(2)));
    RematchVo rematch = RematchVo.suggestedRematch(productList.get(0), suggestedProductSet, MatchReason.INACTIVATION);
    /* Behavior & Verification*/
    assertEquals(false, rematch.getIsAutomaticRematch());
    assertEquals(suggestedProductSet, rematch.getToSuggestedProducts());
    assertEquals(suggestedProductSet, rematch.getMatchProducts());
  }

}
