/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.common.vo.diff.test;


import static org.junit.Assert.assertTrue;

import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.test.generator.CopayTierGenerator;
import gov.va.med.pharmacy.peps.common.vo.CopayTierAssocVo;
import gov.va.med.pharmacy.peps.common.vo.CopayTierVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Diff;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;


/**
 * Verify that a diff between two ValueObjects works.
 */
public class DiffCopayTierTest {
    
    /** The Constant LOG. */
	private static final org.apache.logging.log4j.Logger LOG = org.apache.logging.log4j.LogManager.getLogger(DiffCopayTierTest.class);


//    private List<CopayTierAssocVo> oldCopayTierAssocs;
/** The copay tier list. */
//    private List<CopayTierAssocVo> newCopayTierAssocs;
    private final List<CopayTierVo> copayTierList = new CopayTierGenerator().generate();

    /**
     * setUp.
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @BeforeClass
    public static void before()  {
        
        LOG.debug("\n--------------------" + DiffCopayTierTest.class.getSimpleName() + " Start --------------------");
    }

    /**
     * teardown.
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @AfterClass
    public static void after()  {
        
        LOG.debug("\n--------------------" + DiffCopayTierTest.class.getSimpleName() + " End --------------------");
    }    
    /**
     * Test two equal Copay Tier Association Lists.
     */
    @Test
    public final void testEqual() {
        ProductVo prod1 = new ProductVo();
        prod1.setVaProductName("Product");
        List<CopayTierAssocVo> list1 = createCopayTierAssocVoList1();
        prod1.setCopayTierAssocs(list1);
        for (CopayTierAssocVo copayTierAssocVo : list1) {
            copayTierAssocVo.setProduct(prod1);
        }
        
        ProductVo prod2 = new ProductVo();
        prod2.setVaProductName("Product");
        List<CopayTierAssocVo> list2 = createCopayTierAssocVoList2();
        prod2.setCopayTierAssocs(list2);
        for (CopayTierAssocVo copayTierAssocVo : list2) {
            copayTierAssocVo.setProduct(prod2);
        }
        
        Collection<Difference> differences = Diff.checkForDifferences(prod1, prod2);
        
        LOG.debug("Differences length:" + differences.size());
        for (Difference difference : differences) {
            LOG.debug("\nOld value: " + difference.getOldValueShortString() 
                + "\nNew value: " + difference.getNewValueShortString());            
        }    
        assertTrue("Lists should not have differences!", differences.size() == 0);
    }


    /**
     * Test two Copay Tier Association Lists where they started equal, but the second list ended the last copay tier 
     * and added a new copay tier.
     */
    @Test
    public final void testAddRow() {
        ProductVo prod1 = new ProductVo();
        prod1.setVaProductName("Product");
        List<CopayTierAssocVo> list1 = createCopayTierAssocVoList1();
        prod1.setCopayTierAssocs(list1);
        for (CopayTierAssocVo copayTierAssocVo : list1) {
            copayTierAssocVo.setProduct(prod1);
        }
        
        ProductVo prod2 = new ProductVo();
        prod2.setVaProductName("Product");
        List<CopayTierAssocVo> list2 = createCopayTierAssocVoList3();
        prod2.setCopayTierAssocs(list2);
        for (CopayTierAssocVo copayTierAssocVo : list2) {
            copayTierAssocVo.setProduct(prod2);
        }

        
        Collection<Difference> differences = Diff.checkForDifferences(prod1, prod2);
        
        LOG.debug("Added Row");
        LOG.debug("Differences length:" + differences.size());
        for (Difference difference : differences) {
            LOG.debug("\nOld value: " + difference.getOldValueShortString() 
                + "\nNew value: " + difference.getNewValueShortString());            
        }                        
        
        assertTrue("Lists should have 1 difference!", differences.size() == 1);
    }
    
    /**
     * Test two Copay Tier Association Lists where they started equal, 
     * but the second list removed the last copay tier and removed the end date of the copay tier 
     * in the list before it.
     */
    @Test
    public final void testRemoveRow() {
        ProductVo prod1 = new ProductVo();
        prod1.setVaProductName("Product");
        List<CopayTierAssocVo> list1 = createCopayTierAssocVoList3();
        prod1.setCopayTierAssocs(list1);
        for (CopayTierAssocVo copayTierAssocVo : list1) {
            copayTierAssocVo.setProduct(prod1);
        }
        
        ProductVo prod2 = new ProductVo();
        prod2.setVaProductName("Product");
        List<CopayTierAssocVo> list2 = createCopayTierAssocVoList4();
        prod2.setCopayTierAssocs(list2);
        for (CopayTierAssocVo copayTierAssocVo : list2) {
            copayTierAssocVo.setProduct(prod2);
        }

        
        Collection<Difference> differences = Diff.checkForDifferences(prod1, prod2);
        LOG.debug("Removed Row");
        LOG.debug("Differences length:" + differences.size());
        for (Difference difference : differences) {
            LOG.debug("\nOld value: " + difference.getOldValueShortString() 
                + "\nNew value: " + difference.getNewValueShortString());            
        }                        
        
        assertTrue("Lists should have 1 difference!", differences.size() == 1);
    }
    
    /**
     * Test two Copay Tier Association Lists where they started equal, but the second list ended the last copay tier.  
     * So, same size list and same copay tiers, just the change to an existing record.
     */
    @Test
    public final void testChangeRow_Same() {
        ProductVo prod1 = new ProductVo();
        prod1.setVaProductName("Product");
        List<CopayTierAssocVo> list1 = createCopayTierAssocVoList1();
        prod1.setCopayTierAssocs(list1);
        for (CopayTierAssocVo copayTierAssocVo : list1) {
            copayTierAssocVo.setProduct(prod1);
        }
        
        ProductVo prod2 = new ProductVo();
        prod2.setVaProductName("Product");
        List<CopayTierAssocVo> list2 = createCopayTierAssocVoList5();
        prod2.setCopayTierAssocs(list2);
        for (CopayTierAssocVo copayTierAssocVo : list2) {
            copayTierAssocVo.setProduct(prod2);
        }

        
        Collection<Difference> differences = Diff.checkForDifferences(prod1, prod2);
        LOG.debug("testChangeRow_Same()");
        LOG.debug("Differences length:" + differences.size());
        for (Difference difference : differences) {
            LOG.debug("\nOld value: " + difference.getOldValueShortString() 
                + "\nNew value: " + difference.getNewValueShortString());            
        }                        
        
        assertTrue("Lists should have 1 difference!", differences.size() == 1);
    }   
    
    /**
     * Test two Copay Tier Association Lists where they started equal, but the second list 
     * swapped out the last copay tier record.  So, same size list, but one different copay tier.
     */
    @Test
    public final void testChangeRow_Different() {
        ProductVo prod1 = new ProductVo();
        prod1.setVaProductName("Product");
        List<CopayTierAssocVo> list1 = createCopayTierAssocVoList1();
        prod1.setCopayTierAssocs(list1);
        for (CopayTierAssocVo copayTierAssocVo : list1) {
            copayTierAssocVo.setProduct(prod1);
        }
        
        ProductVo prod2 = new ProductVo();
        prod2.setVaProductName("Product");
        List<CopayTierAssocVo> list2 = createCopayTierAssocVoList6();
        prod2.setCopayTierAssocs(list2);
        for (CopayTierAssocVo copayTierAssocVo : list2) {
            copayTierAssocVo.setProduct(prod2);
        }

        
        Collection<Difference> differences = Diff.checkForDifferences(prod1, prod2);
        LOG.debug("testChangeRow_Different()");
        LOG.debug("Differences length:" + differences.size());
        for (Difference difference : differences) {
            LOG.debug("\nOld value: " + difference.getOldValueShortString() 
                + "\nNew value: " + difference.getNewValueShortString());            
        }                        
        
        assertTrue("Lists should have 1 difference!", differences.size() == 1);
    }      

    /**
     * There was a to-do in {@link Diff#attributesNotEqual(Object, Object)} where there was a problem of an inequality
     * reported between two ManagedItemVo attributes even though they were the "same" (should have been equal, but were two
     * different memory references).
     * <p>
     * Test that two ProductVo clones are shown as equal (they have no differences).
     *
     * @return the list
     */
//    public void testEqualClones() {
//        ProductVo product = new ProductGenerator().getFirst();
//        ProductVo productCopy = product.copy();
//
//        assertFalse("ProductVo should not be the same instance", product == productCopy);
//        assertFalse("VA Dispense Unit should not be the same instance", product.getDispenseUnit() == productCopy
//            .getDispenseUnit());
//
//        Collection<Difference> differences = product.diff(productCopy);
//        LOG.debug("Differences6: \n" + differences);
//        assertTrue("ProductVo should not have differences", differences.size() == 0);
//    }
    
    /**
     * Create a list containing two CopayTierAssocVo - with copay tier 1 and 2.
     * The first entry has an end date and the last one does not.
     * @return the list of CopayTierAssocVo
     */
    private List<CopayTierAssocVo> createCopayTierAssocVoList1(){
        List<CopayTierAssocVo> list = new ArrayList<CopayTierAssocVo>(2);
        CopayTierAssocVo cta1 = new CopayTierAssocVo();
        cta1.setId("1");
        cta1.setCopayTier(copayTierList.get(1));
        cta1.setProduct(new ProductVo());
        
        LocalDate date = LocalDate.now();
        cta1.setEffectiveDate(Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));
        
        date = date.plusDays(29);
        cta1.setEndDate(Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));
        
        list.add(cta1);
        
        CopayTierAssocVo cta2 = new CopayTierAssocVo();
        cta2.setId("2");
        cta2.setCopayTier(copayTierList.get(2));
        cta2.setProduct(new ProductVo());
        
        date = date.plusDays(1);
        cta2.setEffectiveDate(Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));
        
        list.add(cta2);
        
        return list;
    }

    /**
     * Create a list exactly the same as list 1.
     *
     * @return the list
     */
    private List<CopayTierAssocVo> createCopayTierAssocVoList2(){
        List<CopayTierAssocVo> list = new ArrayList<CopayTierAssocVo>(2);
        CopayTierAssocVo cta1 = new CopayTierAssocVo();
        cta1.setId("1");
        cta1.setCopayTier(copayTierList.get(1));
        cta1.setProduct(new ProductVo());
        
        LocalDate date = LocalDate.now();
        cta1.setEffectiveDate(Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));
        
        date = date.plusDays(29);
        cta1.setEndDate(Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));
        
        list.add(cta1);
        
        CopayTierAssocVo cta2 = new CopayTierAssocVo();
        cta2.setId("2");
        cta2.setCopayTier(copayTierList.get(2));
        cta2.setProduct(new ProductVo());
        
        date = date.plusDays(1);
        cta2.setEffectiveDate(Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));
                
        list.add(cta2);          
        
        return list;
    }    
    
    /**
     * Create a list containing three CopayTierAssocVo - with copay tier 1 and 2, and 3.  This mimics adding a row to list 1.
     * All entries have an end date except the last one.
     *
     * @return the list
     */
    private List<CopayTierAssocVo> createCopayTierAssocVoList3(){
        List<CopayTierAssocVo> list = createCopayTierAssocVoList1();
        CopayTierAssocVo cta2 = list.get(1);       
        LocalDate date = cta2.getEffectiveDate().toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        date = date.plusDays(29);
        cta2.setEndDate(Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));
        
        
        CopayTierAssocVo cta3 = new CopayTierAssocVo();
        cta3.setId("3");
        cta3.setCopayTier(copayTierList.get(3));
        cta3.setProduct(new ProductVo());
        
        date = date.plusDays(1);
        cta3.setEffectiveDate(Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));
               
        list.add(cta3);
                                     
        return list;
    }      

    /**
     * Create a list that mimics removing a row from list 3 and reactivating the second row.
     * All entries have an end date except the last one.
     *
     * @return the list
     */
    private List<CopayTierAssocVo> createCopayTierAssocVoList4(){
        List<CopayTierAssocVo> list = createCopayTierAssocVoList3();
        CopayTierAssocVo cta2 = list.get(1);       
        cta2.setEndDate(null);

        list.remove(2);                         
        
        return list;
    }      
    
    /**
     * Create a list containing three CopayTierAssocVo - with copay tier 1 and 2.  
     * This mimics list 1 and adds an end date to the last row.
     *
     * @return the list
     */
    private List<CopayTierAssocVo> createCopayTierAssocVoList5(){
        List<CopayTierAssocVo> list = createCopayTierAssocVoList1();
        CopayTierAssocVo cta2 = list.get(1);       
        LocalDate date = cta2.getEffectiveDate().toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        date = date.plusDays(29);
        cta2.setEndDate(Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));
               
        return list;
    }
    
    /**
     * Create a list similar to List 1 but remove copay tier 2 and replace it with copay tier 3.
     * So, same size list, but the copay tier is different on the last row.
     *   
     *
     * @return the list
     */
    private List<CopayTierAssocVo> createCopayTierAssocVoList6(){
        List<CopayTierAssocVo> list = createCopayTierAssocVoList1();
        CopayTierAssocVo cta2 = list.get(1);              
        
        CopayTierAssocVo cta3 = new CopayTierAssocVo();
        cta3.setId("3");
        cta3.setCopayTier(copayTierList.get(3));
        cta3.setProduct(new ProductVo());                
        cta3.setEffectiveDate(cta2.getEffectiveDate());
               
        list.add(cta3);
        
        list.remove(1);
                                     
        return list;
    }       
}
