package gov.va.med.pharmacy.peps.common.vo.diff.test;

import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.datafield.DataField;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;

/**
 * Generates {@link Difference} objects for use in tests.
 */
public final class DifferenceGenerator {
  
  /**
   * No construction: only static methods.
   */
  private DifferenceGenerator() {
  }

  /**
   * Generates a {@link Difference} for a {@link DataField} field.
   * 
   * @param <T> the type of value that goes in the DataField
   * @param fieldKey the field for the difference
   * @param oldValue the value to put in the old DataField
   * @param newValue the value to put in the new DataField
   * @return never null
   */
  public static <T> Difference makeDifferenceForDataField(
      final FieldKey<DataField<T>> fieldKey, final T oldValue, final T newValue) {

    DataField<T> oldDataField = DataField.newInstance(fieldKey);
    oldDataField.selectValue(oldValue);

    DataField<T> newDataField = DataField.newInstance(fieldKey);
    newDataField.selectValue(newValue);

    return new Difference(fieldKey, oldDataField, newDataField);
  }
}
