package gov.va.med.pharmacy.peps.common.vo.test;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertThat;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;

import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.PPSConstants;
import gov.va.med.pharmacy.peps.common.utility.impl.test.EnvironmentUtilityStub;
import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;
import gov.va.med.pharmacy.peps.common.vo.EventCategory;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryDetailsVo;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryVo;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo;
import gov.va.med.pharmacy.peps.common.vo.NdcVo;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.common.vo.RequestVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;

/**
 * Automated tests for {@link NdcVo} .
 */
public class NdcVoTest {
  
  private EnvironmentUtilityStub environmentUtility;
  
  public NdcVoTest() {
    environmentUtility = new EnvironmentUtilityStub();
    environmentUtility.setNational();
  }
  
  /**
   * Test that {@link ManagedItemVo#determineRematchTriggers(Collection)} returns the correct boolean if a non rematch
   * trigger class is used.
   */
  @Test
  public void testDetermineRematchTriggers() {
  
    final NdcVo objUnderTest = new NdcVo();
    
    /* Setup */
    Difference differenceIncorrectClass =
        new Difference(FieldKey.ITEM_STATUS, PPSConstants.ACTIVE, PPSConstants.INACTIVE);
    
    /* Behavior & Verification */
    assertThat(
        objUnderTest.determineRematchTriggers(Arrays.asList(differenceIncorrectClass)),
        equalTo(Collections.emptySet()));
  }
  
  /**
   * Tests {@link ManagedItemVo#updateFromRequest(RequestVo, UserVo,
   * gov.va.med.pharmacy.peps.common.utility.EnvironmentUtility)}.
   */
  @Test
  public void testUpdateFromRequest() {
    
    /* Setup */
    final String originalVendor = "Original Vendor";
    final String newVendor = "New Vendor";
    final NdcVo objUnderTest = new NdcVo();
    objUnderTest.setId("12222");
    objUnderTest.setItemStatus(ItemStatus.ACTIVE);
    // Set to the new vendor to make sure we roll back.
    objUnderTest.setVendor(newVendor);
    final UserVo user = new UserGenerator().getNationalManagerTwo();
    
    // Make the request.
    final RequestVo request = new RequestVo();
    request.setItemId(objUnderTest.getId());
    final ModDifferenceVo approvedModDifference = new ModDifferenceVo();
    approvedModDifference.setDifference(new Difference(
        FieldKey.ITEM_STATUS, objUnderTest.getItemStatus(), ItemStatus.INACTIVE));
    approvedModDifference.setModificationReason("This is a reason.");
    approvedModDifference.setModRequestItemStatus(RequestItemStatus.APPROVED);
    final ModDifferenceVo rejectedModDifference = new ModDifferenceVo();
    rejectedModDifference.setDifference(new Difference(FieldKey.VENDOR, originalVendor, newVendor));
    rejectedModDifference.setModificationReason("Another reason.");
    rejectedModDifference.setModRequestItemStatus(RequestItemStatus.REJECTED);
    request.setRequestDetails(Arrays.asList(approvedModDifference, rejectedModDifference));
    
    // Make the expected ItemAuditHistoryVo.
    final ItemAuditHistoryVo expectedAuditHistory = new ItemAuditHistoryVo(
        objUnderTest, environmentUtility.getSiteNumber(), EventCategory.SYSTEM_EVENT, user.getUsername());
    // The constructor should probably set these, but not changing that now.
    expectedAuditHistory.setReason("");
    expectedAuditHistory.setOldValue("");
    expectedAuditHistory.setNewValue("");
    final ItemAuditHistoryDetailsVo expectedRejectedDetails = new ItemAuditHistoryDetailsVo(
        objUnderTest, rejectedModDifference, environmentUtility.getSiteNumber(), EventCategory.REJECTED_REQUEST,
        user.getUsername());
    expectedRejectedDetails.setDetailReason(rejectedModDifference.getModificationReason());
    expectedAuditHistory.addDetail(expectedRejectedDetails);
    final ItemAuditHistoryDetailsVo expectedApprovedDetails = new ItemAuditHistoryDetailsVo(
        objUnderTest, approvedModDifference, environmentUtility.getSiteNumber(), EventCategory.APPROVED_REQUEST,
        user.getUsername());
    expectedApprovedDetails.setDetailReason(approvedModDifference.getModificationReason());
    expectedAuditHistory.addDetail(expectedApprovedDetails);
    
    /* Behavior & Verification */
    objUnderTest.updateFromRequest(request, user, environmentUtility);
    
    assertThat(objUnderTest.getNewAuditHistory(), equalTo(expectedAuditHistory));
    assertThat(objUnderTest.getItemStatus(), equalTo(ItemStatus.INACTIVE));
    // Should have revereted vendor to original value.
    assertThat(objUnderTest.getVendor(), equalTo(originalVendor));
  }
    
}
