package gov.va.med.pharmacy.peps.domain.common.capability;

import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.model.EplCopayTierDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProdCopayTierAssocDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.ProductConverter;

/**
 * Domain-level operations supporting copay tier migration.
 */
public interface CopayTierMigrationDomainCapability {


  /**
   * Retrieve copay tier assoc records by product.
   * @param eplProductDo EplProductDo
   * @return List<EplProdCopayTierAssocDo>
   */
  List<EplProdCopayTierAssocDo> retrieveCopayTierAssocs(EplProductDo eplProductDo);
  
  /**
   * Retrieves all defined copay tiers.
   * 
   * @return all copay tiers in no particular order, or null if unable to retrieve them.
   */
  List<EplCopayTierDo> retrieveCopayTiers();
  
  /**
   * Retrieve a product by VUID.
   * 
   * @param vuid the VA unique identifier for the product to retrieve
   * @return EplProductDo the product with the matching VUID, or null if no matching product is found.
   */
  EplProductDo retrieveProduct(int vuid);
  
  /**
   * Get SeqNumDomainCapability.
   *  
   * @return SeqNumDomainCapability
   */
  SeqNumDomainCapability getSeqNumDomainCapability();
  
  /**
   * Get the {@link ProductConverter} to use.
   *  
   * @return the instance
   */
  public ProductConverter getProductConverter();
  
  /**
   * Insert copay tier association record.
   *  
   * @param eplProdCopayTierAssocDo EplProdCopayTierAssocDo
   * @param user UserVo
   */
  public void insertCopayTierAssoc(EplProdCopayTierAssocDo eplProdCopayTierAssocDo, UserVo user);
  
  /**
   * Update copay tier association record.
   *  
   * @param eplProdCopayTierAssocDo EplProdCopayTierAssocDo
   * @param user UserVo
   */
  public void updateCopayTierAssoc(EplProdCopayTierAssocDo eplProdCopayTierAssocDo, UserVo user);
  
  /**
   * Delete copay tier association record by passing in eplProdCopayTierAssocDo object.
   *  
   * @param eplProdCopayTierAssocDo EplProdCopayTierAssocDo
   */
  public void deleteCopayTierAssoc(EplProdCopayTierAssocDo eplProdCopayTierAssocDo);
  
  /**
   * Delete copay tier association record by field name and value.
   *  
   * @param fieldName String
   * @param value Object
   */
  public void deleteCopayTierAssocs(String fieldName, Object value);
}
