package gov.va.med.pharmacy.peps.domain.common.capability;

import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.common.vo.PaginatedList;
import gov.va.med.pharmacy.peps.common.vo.UpdateSearchCriteria;
import gov.va.med.pharmacy.peps.common.vo.UserVo;

import java.util.List;

/**
 * Perform CRUD operations on {@link NdfUpdateFileVo}s.
 */
public interface NdfUpdateFileMgtDomainCapability {
    
    /**
     * Retrieves all NdfUpdateFileVo
     * 
     * @return Collection of NdfUpdateFileVo
     */
    List<NdfUpdateFileVo> retrieveAll();
    
    /**
     * Retrieves pending NdfUpdateFileVo
     * 
     * @return Collection of NdfUpdateFileVo
     */
    List<NdfUpdateFileVo> retrievePending();
    
    /**
     * Retrieves NdfUpdateFileVo
     * 
     * @return NdfUpdateFileVo
     */
    NdfUpdateFileVo retrieveById(Long id);
    
    /**
     * Updates the NDF update
     * 
     * @param NdfUpdateFileVo   updateFile
     *   
     * @param user
     *            provides user information
     * 
     */
    void update(NdfUpdateFileVo updatedFile, UserVo user);
    
    /**
     * Retrieves initiated NdfUpdateFileVo
     * 
     * @return Collection of NdfUpdateFileVo
     */
    List<NdfUpdateFileVo> retrieveInitiated();
    
    /**
     * Inserts the initiated NDF update
     * 
     * @param NdfUpdateFileVo   updateFile
     *   
     * @param user
     *            provides user information
     * 
     */
    NdfUpdateFileVo insert(NdfUpdateFileVo initiatedFile, UserVo user);
    
    /**
     * Retrieves the last NDF update transmitted to production 
     * 
     */
    NdfUpdateFileVo retrieveLastTranmittedToProduction();
    
     /**
     * Retrieves records in initiated status
     * 
     */
    NdfUpdateFileVo checkInitiatedFile();
    
    /**
     * Retrieves the latest NdfUpdateFileVo, meaning the one with the max file id.
     *
     * @return the NdfUpdateFileVo with the highest file id
     */
    NdfUpdateFileVo retrieveCurrentStatus();
    
    PaginatedList<NdfUpdateFileVo> retrievePaged(UpdateSearchCriteria searchCriteria);


}
