package gov.va.med.pharmacy.peps.domain.common.capability;

import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.UnmatchRematchVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;

import java.util.List;
import java.util.Optional;

/**
 * Performs CRUD operations on {@link RematchVo}s.
 */
public interface RematchMgtDomainCapability {
    
    /**
     * Updates the given rematch in the database.
     * @param rematch the rematch to update. May or may not have previously been saved to the database.
     * @param user the user updating it
     * @return the updated rematch from the database
     */
    public RematchVo saveOrUpdate(RematchVo rematch, UserVo user);
    
    /**
     * Deletes the given rematch from the database.
     * @param rematch the rematch to delete
     */
    public void delete(RematchVo rematch);
    
    /**
     * This method should be removed under Story 452145.
     */
    @Deprecated
    public void removeObsoleteProducts();
    
    /**
     * This method should be removed under Story 452145.
     */
    @Deprecated
    public void addUnmatchedProducts(UserVo user);

    /**
     * Returns the {@link RematchVo}s for a given matchType that have not been sent in an update file.
     * @param matchTypeId the ID for the given matchType
     */
    public List<RematchVo> retrieveUnmatched(Long matchTypeId);

    /**
     * Returns a list of potential matches for a product.
     * @param productId the id of the product to match {@link ProductVo#getId()}
     * @return the list of potential matches in arbitrary order, can be zero or more.
     */
    public List<UnmatchRematchVo> findRematchProducts(Long productId);

    /**
     * Returns the unsent rematch, if any, for the given product.
     * @param productId the product's database ID
     * @return an Optional containing the rematch if one exists. Never null.
     */
    public Optional<RematchVo> getRematchForProduct(long productId);

}
