package gov.va.med.pharmacy.peps.domain.common.dao;


import java.util.Date;
import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.service.common.update.CopayTierChange;

/**
 * Data Access Object for {@link EplNdfOutgoingDifferences} objects.
 */
public interface EplNdfOutgoingDifferencesDao {

    /**
     * Find by id.
     *
     * @param id the id
     * @return the epl ndf outgoing differences
     */
    public abstract EplNdfOutgoingDifferences findById(Long id);

    /**
     * Insert a new EplNdfOutgoingDifferences record.
     *
     * @param eplNdfOutgoingDifferences the epl ndf outgoing differences
     * @return the epl ndf outgoing differences
     */
    public abstract EplNdfOutgoingDifferences save(EplNdfOutgoingDifferences eplNdfOutgoingDifferences);
    
    /**
     * Update the specified existing EplNdfOutgoingDifferences record.
     *
     * @param eplNdfOutgoingDifferences the record to update
     * @return the EplNdfOutgoingDifferences record after the update.
     */
    public abstract EplNdfOutgoingDifferences update(EplNdfOutgoingDifferences eplNdfOutgoingDifferences);
    
    /**
     * update the ndf update_file_fk_field with the generated output file seq number.
     *
     * @param returnedDifference List<EplNdfOutgoingDifferences> to associate with the specified file
     * @param ndfUpdateFile The NDF Update File to associate with the specified outgoing differences
     */
    
    public void updateNdfUpdateFileFk(List<EplNdfOutgoingDifferences> returnedDifference, NdfUpdateFileVo ndfUpdateFile);
    
    /**
     * set the ndf update_file_fk_field to null to remove association.
     *
     * @param fileSeqNo Id of the NDF Update File to clear from outgoing differences.
     */
    
    public void removeNdfUpdateFileFk(Long fileSeqNo);
    
    /**
     * Find updates that have not yet been included in an updated file (has no file FK associated).
     *
     * @param startDate the start date
     * @return eplNdfOutgoingDifferences the epl ndf outgoing differences
     */
    public List<EplNdfOutgoingDifferences> retrieveDifferencesForOutputFile(Date startDate);
    
    /**
     * Find by vista file by vista file, field, Ien where data has not been included in an output file yet.
     *
     * @param vistaFileNumber the vista file number
     * @param vistaFieldNumber the vista field number
     * @param vistaIen the vista ien
     * @return eplNdfOutgoingDifferences the outgoing differences
     */
    public EplNdfOutgoingDifferences findByFileFieldIen(String vistaFileNumber, String vistaFieldNumber, String vistaIen);

    /**
     * Find by VistA file and field where data has not been included in an update file. Only returns modify actions.
     *
     * @param vistaFile the file number to search for
     * @param vistaField the VistA field number to search for
     * @param startDTM the inclusive upper date/time bound of changes to find
     * @return an array of returned columns directly from the database query
     */
    public List<Object[]> findByFileAndField(String vistaFile, String vistaField, Date startDTM);
    
    /**
     * Find inactive generic name in an update file.
     *
     * @param vistaFileNumber Must always be 50.6. (This parameter should be removed.)
     * @param vistaFieldNumber the field number of the changed VistA field
     * @param startDTM the inclusive upper date/time bound of changes to find
     * @return String the vista Ien
     */
    public List<String> findInactiveGenericName(String vistaFileNumber, String vistaFieldNumber, Date startDTM);
    
    /**
     * Find inactive generic name in an update file.
     *
     * @param vistaFileNumber the vista file number
     * @param vistaFieldNumber the vista field number
     * @param startDTM the inclusive upper date/time bound of changes to find
     * @return the list
     */
    public List<Object[]> findOverrideDFChkExcl(String vistaFileNumber, String vistaFieldNumber, Date startDTM);
 
    /**
     * Find drug class changes by vista file and field where data has not been included in an update file.
     *
     * @param vistaFileNumber Must always be 50.68. (This parameter should be removed.)
     * @param vistaFieldNumber the field number of the changed VistA field
     * @param startDTM the inclusive upper date/time bound of changes to find
     * @return List the drug changes
     */   
    public List<Object[]> findDrugClassChanges(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findActiveNatFormularyProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findInactiveNatFormularyProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    /**
     * Returns existing Products that had their CMOP Dispense Indicator modified (Action 'M') to unchecked ('1') but it is a 
     * pending outgoing difference.
     *
     * @param startDTM the start date of the reporting process requesting this data such that any newer data is not included
     * @return the list of Object arrays containing the rows and fields from the query result
     */
    public List<Object[]> findInactivatedCMOPProducts(Date startDTM);

    public List<Object[]> findAddedProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findNewClassProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    /**
     * Find pending records (records that have not been included in a file yet) with the specified Vista IEN, VistA file  
     * number, VistA field number, and action type.  Other code manages the pending records such that there should only ever 
     * be one pending record at a time for a given file/field, which is why this only returns one record instead of a list. 
     *
     * @param vistaFileNumber the VistA file number
     * @param vistaFieldNumber the VistA field number
     * @param vistaIen the VistA IEN
     * @param actionType the action type
     * @return the EplNdfOutgoingDifferences or null if none found
     */
    public EplNdfOutgoingDifferences findModifiedRecords(String vistaFileNumber, String vistaFieldNumber, 
        String vistaIen, String actionType);

    public List<Object[]> findUnmatchedProducts(String vistaFileNumber);

    public List<Object[]> findScheduleChangeProducts(String vistaFileNumber, String vistaFieldNumber);

    public List<Object[]> findNoPossibleDosageProducts(String vistaFileNumber, Date startDTM);

    public List<Object[]> findYesPossibleDosageProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findAddedMedGuideProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findEditedMedGuideProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findExcludedDDIProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findDeletedMedGuideProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> messageGenericChange(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> dispenseUnitsChange(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> drugUnitsChange(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> classChange(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    /**
     * Find products to be in-activated in the future and don't have a rematch.
     *     
     * @return report data list
     */
    public List<Object[]> scheduleAllChange(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> genNameAllChange(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> dosageFormChange(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findPrevExcludedDDIProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findReactivatedProducts(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    /**
     * Find products made inactive at local site to add to PRODUCT^ list.
     * @param vistaFileNumber Must always be 50.68. (This parameter should be removed.)
     * @param startDTM the inclusive upper date/time bound of changes to find
     * @return an array of returned columns directly from the database query
     */
    public List<Object[]> findInactivatedProductsForLocal(String vistaFileNumber, Date startDTM);

    /**
     * Find inactivated products that don't have a rematch.
     * @param startDTM the start dtm
     * @return an array of returned columns directly from the database query
     */
    public List<Object[]> findInactivatedProducts(Date startDTM);

    public List<Object[]> findAddedDDIs(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findEditedDDIs(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findInactivatedDDIs(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    public List<Object[]> findReactivatedDDIs(String vistaFileNumber, String vistaFieldNumber, Date startDTM);

    /**
     * Returns existing Products that had their CMOP Id modified (Action 'M') but it is a pending outgoing difference.
     *
     * @param startDTM the start date of the reporting process requesting this data such that any newer data is not included 
     * @return the list of Object arrays containing the rows and fields from the query result
     */
    public List<Object[]> findCMOPIdChanges(Date startDTM);

  /**
   * <p>
   * Find products with copay tier changes.
   * </p>
   * 
   * <p>
   * This method deviates from the pattern in this class by not including the unnecessary file and field number parameters.
   * </p>
   *
   * @param startDTM the start time of the update file generation process
   * @return a list of CopayTierChange objects, one for each outgoing copay tier record. The list will be sorted by product
   *         name, then copay tier index.
   */
  public List<CopayTierChange> findCopayTierChanges(Date startDTM);

  /**
   * Removes all pending outgoing differences (that have not been associated to a file yet) for the specified file, multiple
   * field number and vistaIen. All fields with the VistA field number base would be included in the delete. Ex. For Copay Tier,
   * the caller would specify 45. Fields [45,.01], [45,1], and [45,2] would be deleted for the vistaIen specified.
   * 
   * The VistA IEN specified should be the main entity's ien. For example, a Copay Tier is associated to a product. THe Vista
   * IEN on the outgoing differences file is a comma-separated aggregation of the product's IEN and the pseudo IEN of the Copay
   * Tier (Ex. [123456,1], [123456,2],...). The caller should specify just the 123456 in this case.
   * 
   * @param vistaFileNumber
   *          VistA File Number
   * @param vistaFieldNumberBase
   *          Base of the VistA Field Number of the multiple field. Ex. Copay Tier would be 45.
   * @param vistaIen
   *          VistA IEN to which the record is associated.
   * 
   * @return int containing the number of records deleted
   */
  public int removePendingOutgoingDifferencesForMultipleField(String vistaFileNumber, String vistaFieldNumberBase, 
      String vistaIen);

  /**
   * Delete the given EplNdfOutgoingDifferences record.
   * 
   * @param diff EplNdfOutgoingDifferences eplNdfOutgoingDifferences instance to delete
   */
  void delete(EplNdfOutgoingDifferences diff);

  /**
   * Find pending records (records that have not been included in a file yet) with the specified Vista IEN, VistA file number
   * and VistA field number.
   *
   * @param vistaFileNumber the VistA file number
   * @param vistaFieldNumber the VistA field number
   * @param vistaIen the VistA IEN
   * @return the list containing any EplNdfOutgoingDifferences that were found
   */
  List<EplNdfOutgoingDifferences> findPendingRecordsForFieldAndIen(String vistaFileNumber, String vistaFieldNumber, 
      String vistaIen);

  public List<Object[]> findFormularyDesignatorProducts(Date startDTM);

}
