/**
 * Source file created in 2011 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.domain.common.dao;


import java.util.Date;
import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.ReportCopayTierChangeVo;
import gov.va.med.pharmacy.peps.common.vo.ReportFdbCopayChangeVo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProdCopayTierAssocDo;


/**
 * This class provides a set of operation that may be performed on all the classes that inherit from this class. This
 * interface should never be used directly, but it will be inherited by all DAOs. It provides abstract methods to access and
 * modify the EplVaGenericNameDo.
 * 
*/
public interface EplProdCopayTierAssocDao extends DataAccessObject<EplProdCopayTierAssocDo, Long> {

    /**
     * Save an instance of the copay tier association.
     * 
     * @param copayProductTier EplProdCopayTierAssocDo
     * @return EplProdCopayTierAssocDo
    */
    public EplProdCopayTierAssocDo save(EplProdCopayTierAssocDo copayProductTier);

    /**
     * Find EplProdCopayTierAssocDo object for a given id.
     * 
     * @param id Long
     * @return EplProdCopayTierAssocDo
    */
    EplProdCopayTierAssocDo findById(Long id);
    
    /**
     * Get all instances of the given DataObject without any particular order.
     * 
     * @return List of DataObjects
    */
    List<EplProdCopayTierAssocDo> retrieve();


    /**
     * Return all DataObjects that have a field equal to the given value.
     * 
     * @param fieldName String name of the field to find by
     * @param value Object value of the field results must be equal to
     * @return List of DataObjects
    */
    List<EplProdCopayTierAssocDo> retrieve(String fieldName, Object value);

    /**
     * Returns the data to be included in the 'FDB Copay Change' report.
     * 
     * @param includeInactiveProducts If true, report will include inactive products. Otherwise,
     * inactive products will be excluded.
     * @param includeNoGcnseqno If true, report will include products in PPS-N which do not have 
     * a GCN sequence number. Otherwise, these products will be excluded.
     * @param includeFederalLegendCode If true, report will include products whose matching FDB
     * records have a federal legend code other than 'O' or 'F'. Otherwise, only products with
     * federal legend code 'O' or 'F' will be included; all other values will be excluded.
     * @return A list of ReportFdbCopayChangeVo objects comprising the data for the report.
     */
    List<ReportFdbCopayChangeVo> retrieveFdbCopayChangeData(
        boolean includeInactiveProducts, 
        boolean includeNoGcnseqno,
        boolean includeFederalLegendCode);

    /**
     * Retrieve the data to be included in the 'Products with Copay Tier Level Changes' report.
     *
     * @param startDate the start date.  If specified, only products that had copay changes between the start date and the 
     *    current date will be retrieved.
     * @param includeActiveProducts the include active products.  Value of 1 = products with an item status of active will 
     *    be included.  Value of 0 = products with an item status of active will NOT be included. 
     * @param includeInactiveProducts the include inactive products.  Value of 1 = products with an item status of inactive  
     *    will be included.  Value of 0 = products with an item status of inactive will NOT be included.
     * @param drugClassDelimiter the drug class delimiter.  Since the drug class list is returned in one field, this delimiter
     *    will be used to separate drug classes. 
     * @return the list of ReportCopayTierChangeVo that match the specified criteria.
     */
    List<ReportCopayTierChangeVo> retrieveProductsWithCopayChangeData(Date startDate, int includeActiveProducts,
        int includeInactiveProducts, String drugClassDelimiter);
}
