package gov.va.med.pharmacy.peps.domain.common.dao;


import java.util.Date;
import java.util.List;
import java.util.Optional;

import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.common.vo.UnmatchRematchVo;
import gov.va.med.pharmacy.peps.domain.common.model.EplRematchDo;


public interface EplRematchDao extends DataAccessObject<EplRematchDo, Long>{

    /**
     * Returns the unsent rematch, if any, for the given product.
     * @param productId the product's database ID
     * @return an Optional containing the rematch if one exists. Never null.
     */
    public Optional<EplRematchDo> retrieveForProduct(long productId);

    public List<Object[]> removeObsoletePrintName();

    public List<Object[]> removeObsoleteGenericName();

    public List<Object[]> removeObsoleteCMOP();
 
    public List<Object[]> removeObsoleteDispense();

    public List<Object[]> removeObsoleteDosageForm();

    public List<Object[]> removeObsoleteStrength();

    public List<Object[]> removeObsoleteDrugUnit();

    public List<Object[]> removeObsoleteFedSch();

    public List<Object[]> loadUnmatchGenericName();

    public List<Object[]> loadUnmatchCMOP();

    public List<Object[]> loadUnmatchDispense();

    public List<Object[]> loadUnmatchPrintName();

    public List<Object[]> loadUnmatchDosageForm();

    public List<Object[]> loadUnmatchStrength();

    public List<Object[]> loadUnmatchDrugUnit();

    public List<Object[]> loadUnmatchFedSch();

    /**
     * Find "other change" automatic rematches that have not been in an update file.
     * 
     * @param startDTM the start date of the update file process
     * @return an array of returned columns directly from the database query
     */
    public List<Object[]> findRematchAuto(Date startDTM);

    /**
     * Finds the automatic and suggested rematches for product inactivation that are ready to send in an update file.
     * @param startDTM the start date of the reporting process requesting this data such that any newer data is not included 
     *
     * @return the rematches. Never null.
     */
    public List<EplRematchDo> findRematchesForInactivations(Date startDTM);   

    /**
     * Returns the {@link EplRematchDo}s for a given matchType that have not been sent in an update file.
     * @param matchTypeId the ID for the given matchType
     */
    public List<EplRematchDo> retrieveByMatchType(Long matchTypeId);

    public List<UnmatchRematchVo> findRematchProducts(Long productId);

    public List<Object[]> findRematchSugg(Date startDTM);

    public abstract void updateRematchNdfUpdateFileFk(Date startDTM, NdfUpdateFileVo ndfUpdateFileVo);

    public List<Object[]> findRematchNoSugg(Date startDTM);

    public abstract void removeNdfUpdateFileFk(Long fileSeqNo);

    public List<Object[]> removeObsoleteOthers();
    
    
}