package gov.va.med.pharmacy.peps.domain.common.dao.impl;

import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator.validateLongInput;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.common.vo.UnmatchRematchVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplRematchDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplRematchDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.NdfUpdateFileConverter;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;


/**
 * Primary implementation of {@link EplRematchDao}.
 */
//@Repository
public class EplRematchDaoImpl extends DataAccessObjectImpl<EplRematchDo, Long>
  implements EplRematchDao {
  
  private static final Logger LOG = LogManager.getLogger(EplRematchDaoImpl.class);
      
    NdfUpdateFileConverter ndfUpdateFileConverter;    

    @Override
    public Optional<EplRematchDo> retrieveForProduct(long productId) {
      LOG.debug("Retrieving rematch for product {}.", productId);

      final Session session = getCurrentSession();

      final Criteria criteria = session.createCriteria(EplRematchDo.class)
          // Null update file means we haven't sent the rematch yet.
          .add(Restrictions.isNull("ndfUpdateFile"))
          .createCriteria("fromProduct").add(Restrictions.eq("eplId", productId));
      // The Hibernate API returns only Object, sadly, but it should always be an EplRematchDo.
      // Note that uniqueResult will throw an Exception if the query returns more than one result.
      final EplRematchDo rematchDo = (EplRematchDo)criteria.uniqueResult();

      return Optional.ofNullable(rematchDo);
    }
  
    @Override
    public List<EplRematchDo> retrieveByMatchType(Long matchId){         
        Long validatedMatchId = null;
        if (matchId != null) {
          validatedMatchId = validateLongInput(matchId.toString());
        }
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("retrieveByMatch").setLong("matchTypeId", validatedMatchId);
            
        List<EplRematchDo> returnedValues = query.list();
        
        return returnedValues;
    }
    
    @Override   
    public List<UnmatchRematchVo> findRematchProducts(Long productId) {
      LOG.debug("Finding rematch products for product {}.", productId);

        Session session = getCurrentSession();
        Query query = session.getNamedQuery("findRematch")
            .setLong("unmatchedProductId",productId);
        List<?> returnedRows = query.list();
        List<UnmatchRematchVo> potentialMatches = new ArrayList<UnmatchRematchVo>();
        for (Object returnedRow : returnedRows) {
          final Object[] potentialMatch = (Object[]) returnedRow;
          UnmatchRematchVo possibleUmatchRematch = new UnmatchRematchVo();
          // The numbers present here represent the columns returned in the returnedColumns query. Must match columns in
          // the "findRematch" named native query.
          possibleUmatchRematch.setMatchedProductId(((BigDecimal) potentialMatch[0]).longValue());
          possibleUmatchRematch.setMatchedProductName((String) potentialMatch[1]);
          possibleUmatchRematch.setMatchedCmop((String) potentialMatch[2]);
          possibleUmatchRematch.setMatchedIen(((BigDecimal) potentialMatch[3]).toString());
          // The SQL fetches only active items for rematch now, set status to active.
          possibleUmatchRematch.setItemStatus(ItemStatus.ACTIVE);
          potentialMatches.add(possibleUmatchRematch);
        }
        
        return potentialMatches;
    } 

   /**
    * Update
    */
    public EplRematchDo update(EplRematchDo eplRematchSuggestion) {
        
        Session session = getCurrentSession();
        session.update(eplRematchSuggestion);
        
        return eplRematchSuggestion;
    }
    
    @Override
    public List<Object[]> removeObsoletePrintName() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("obsoleteUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "5");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> removeObsoleteOthers() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("obsoleteUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68");            
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> loadUnmatchPrintName() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("loadUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "5");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> removeObsoleteGenericName() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("obsoleteUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", ".05");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> loadUnmatchGenericName() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("loadUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", ".05");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> removeObsoleteCMOP() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("obsoleteUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "6");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> loadUnmatchCMOP() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("loadUnmatchCmop")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "6");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> removeObsoleteDispense() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("obsoleteUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "8");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> loadUnmatchDispense() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("loadUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "8");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> removeObsoleteDosageForm() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("obsoleteUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "1");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> loadUnmatchDosageForm() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("loadUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "1");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> removeObsoleteStrength() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("obsoleteUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "2");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> loadUnmatchStrength() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("loadUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "2");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> removeObsoleteDrugUnit() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("obsoleteUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "3");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> loadUnmatchDrugUnit() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("loadUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "3");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> removeObsoleteFedSch() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("obsoleteUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "19");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @Override
    public List<Object[]> loadUnmatchFedSch() {
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("loadUnmatch")
            .setInteger("matchTypeId", 3)
            .setString("vistaFileNumber","50.68")
            .setString("vistaFieldNumber", "19");
            
        List<Object[]> returnedValues = query.list(); 
        
        return returnedValues;
    }
    
    @SuppressWarnings("unchecked")
    @Override
    public List<Object[]> findRematchAuto(Date startDTM){        
        Timestamp startDate = new Timestamp(startDTM.getTime());   
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("rematchAuto")  
          .setTimestamp("startDate", startDate)
          .setReadOnly(true);
        
        return (List<Object[]>) query.list();
   } 

  @Override
  public List<EplRematchDo> findRematchesForInactivations(Date startDTM) {
    
    LOG.debug("Querying for rematches for inactivations.");

    Timestamp startDate = new Timestamp(startDTM.getTime());
    Session session = getCurrentSession();
    Query query = session.getNamedQuery("inactiveRematchReport")
        .setString("vistaFileNumber", VistaProductFile.FILE_NUMBER)
        .setString("vistaFieldNumber", VistaProductFile.STATUS_FIELD_NUMBER)
        .setTimestamp("startDate", startDate);
    
    @SuppressWarnings("unchecked") // Assumption: the HQL query actually returns EplRematchDos.
    final List<EplRematchDo> list = query.list();

    return list;
  }
    
    /**
     * Find rematch suggestion
     *
     * 
     * @return report data list
     */
    @SuppressWarnings("unchecked")
    @Override
    public List<Object[]> findRematchSugg(Date startDTM){        
        Timestamp startDate = new Timestamp(startDTM.getTime());   
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("rematchSuggestion")  
          .setTimestamp("startDate", startDate)
          .setReadOnly(true);
        
        return (List<Object[]>) query.list();
   }   
    
    /**
     * Find rematch suggestion
     *
     * 
     * @return report data list
     */
    @SuppressWarnings("unchecked")
    @Override
    public List<Object[]> findRematchNoSugg(Date startDTM){        
        Timestamp startDate = new Timestamp(startDTM.getTime());   
        Session session = getCurrentSession();
        Query query = session.getNamedQuery("rematchNoSuggestion")  
          .setTimestamp("startDate", startDate)
          .setReadOnly(true);
        
        return (List<Object[]>) query.list();
   }  
    
    @Override
    public void updateRematchNdfUpdateFileFk(Date startDTM, NdfUpdateFileVo ndfUpdateFileVo){        

        // Assumption: only this query was used to put rematches into the file. This assumption is true as long as we
        // are only sending rematches for inactivations.
        List<EplRematchDo> returnedValues = findRematchesForInactivations(startDTM);
                
        for (EplRematchDo eplRematchDo : returnedValues) {
            
            eplRematchDo.setNdfUpdateFile(ndfUpdateFileConverter.convert(ndfUpdateFileVo));
            update(eplRematchDo);
        }
        
        
   }
    
    @Override
    public void removeNdfUpdateFileFk(Long fileSeqNo) {
        
        Session session = getCurrentSession();
        Query query = session.createQuery("update EplRematchDo set ndfUpdateFile = null "
            + " where ndfUpdateFile = " + fileSeqNo);
        query.executeUpdate();
    }
    
    public NdfUpdateFileConverter getNdfUpdateFileConverter() {
        return ndfUpdateFileConverter;
    }

    
    public void setNdfUpdateFileConverter(NdfUpdateFileConverter ndfUpdateFileConverter) {
        this.ndfUpdateFileConverter = ndfUpdateFileConverter;
    }   

}
    
    
