package gov.va.med.pharmacy.peps.domain.common.model;


import java.util.Date;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;


/**
 * This class models formulary designator data objects.
 */
public class EplFormularyDesignatorDo extends
        gov.va.med.pharmacy.peps.domain.common.model.DataObject implements
        java.io.Serializable {


    /**
     * Constant with the string value of the EPL_ID field name.
     */
    public static final String EPL_ID = "eplId";
    
    /**
     * Constant with the string value of the NAME field name.
     */
    public static final String NAME = "name";
    
    /**
     * Constant with the string value of the ITEM_STATUS field name. 
     */
    public static final String ITEM_STATUS = "itemStatus";
    
    /**
     * Constant with the string value of the DESCRIPTION field name. 
     */
    public static final String DESCRIPTION = "description";
    
    /**
     * Constant with the string value of the INACTIVATION_DATE field name. 
     */
    public static final String INACTIVATION_DATE = "inactivationDate";

    private static final long serialVersionUID = 1L;

    private Long eplId;
    private String name;
    private String description;
    private String itemStatus;
    private Date inactivationDate;
    private Long revisionNumber;


    /** 
     * Default constructor.
     */
    public EplFormularyDesignatorDo() {
    }

    /**
     * Minimal constructor.
     * 
     * @param id id
     * @param name name
     * @param description description
     * @param itemStatus itemStatus
     * @param requestStatus requestStatus
     * @param inactivationDate inactivationDate
     * @param createdBy createdBy
     * @param createdDtm createdDtm
     */
    public EplFormularyDesignatorDo(Long id, String name, String description, String itemStatus,
        String requestStatus, Date inactivationDate, String createdBy, Date createdDtm) {
        this.eplId = id;
        this.name = name;
        this.description = description;
        this.itemStatus = itemStatus;
        this.inactivationDate = inactivationDate;
        this.setCreatedBy(createdBy);
        this.setCreatedDtm(createdDtm);
    }

    /**
     * Full constructor.
     * 
     * @param id id
     * @param name name
     * @param description description
     * @param itemStatus itemStatus
     * @param requestStatus requestStatus
     * @param inactivationDate inactivationDate
     * @param createdBy createdBy
     * @param createdDtm createdDtm
     * @param lastModifiedBy lastModifiedBy
     * @param lastModifiedDtm lastModifiedDtm
     */
    public EplFormularyDesignatorDo(Long id, String name, String description, String itemStatus,
        String requestStatus, Date inactivationDate, String createdBy, Date createdDtm,
        String lastModifiedBy, Date lastModifiedDtm) {
        this.eplId = id;
        this.name = name;
        this.description = description;
        this.itemStatus = itemStatus;
        this.inactivationDate = inactivationDate;
        this.setCreatedBy(createdBy);
        this.setCreatedDtm(createdDtm);
        this.setLastModifiedBy(lastModifiedBy);
        this.setLastModifiedDtm(lastModifiedDtm);
    }
   
    public Long getEplId() {
        return this.eplId;
    }

    public void setEplId(Long eplId) {
        this.eplId = eplId;
    }

    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }
    
    public String getDescription() {
        return this.description;
    }
    public void setDescription(String description) {
        this.description = description;
    }
   
   public String getItemStatus() {
       return this.itemStatus;
   }
   
   public void setItemStatus(String itemStatus) {
       this.itemStatus = itemStatus;
   }
     
   public Date getInactivationDate() {
       return this.inactivationDate;
   }
   
    public void setInactivationDate(Date inactivationDate) {
        this.inactivationDate = inactivationDate;
    }    

    
    /**
     * Get the revision number.
     * 
     * @return the revisionNumber
     */
    public Long getRevisionNumber() {
        return revisionNumber;
    }

    
    /**
     * Set the revision number.
     * 
     * @param revisionNumber the revisionNumber to set
     */
    public void setRevisionNumber(Long revisionNumber) {
        this.revisionNumber = revisionNumber;
    }

    /**
     * toString returned by Jakarta Commons ToStringBuilder.
     * 
     * @return String properties and values of the ValueObject
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
    }    
    
    @Override
    public boolean equals(Object other) {
        if ((this == other)) {
            return true;
        }

        if ((other == null)) {
            return false;
        }

        if (!(other instanceof EplFormularyDesignatorDo)) {
            return false;
        }

        EplFormularyDesignatorDo castOther = (EplFormularyDesignatorDo) other;

        return ((this.getEplId() == castOther.getEplId()) || (this
                .getEplId() != null
                && castOther.getEplId() != null && this
                .getEplId().equals(castOther.getEplId()))) 
                && ((this.getEplId() == castOther.getEplId()) 
                        || (this.getEplId() != null
                && castOther.getEplId() != null && this.getEplId().equals(
                    castOther.getEplId())));
    }

    @Override
    public int hashCode() {
        int result = 17;
        result = 37 * result + (getEplId() == null ? 0 : this.getEplId().hashCode());
        return result;
    }

}
