package gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;

import gov.vha.isaac.rest.api1.data.search.RestSearchResult;
import gov.vha.isaac.rest.api1.data.search.RestSearchResultPage;
import gov.vha.isaac.rest.api1.data.sememe.RestDynamicSememeVersion;


/**
 * This class handles the process of keeping RxNorm data up to date on the EPL_PRODUCTS table.
 * It uses two web service calls to NLM and the product DAO to populate the data into the appropriate columns.
 * 
 * @author D. Caruso
 *
 */
public class RxNormClient {
	
	private String codingConceptURL;
    private String codingComponentURL;
  
    /**
     * Calls the ISAAC web service passing a product VUID and returning the Code System Code if the web service passes a result
     * 
     * @param vuid
     * @return String
     */
    public String getCode(String rxcuid){
      if (rxcuid != null){
        Map<String, String> params = new HashMap<String, String>();
        params.put("VUID", rxcuid);
        RestTemplate restTemplate = new RestTemplate();
        restTemplate.setRequestFactory(new HttpComponentsClientHttpRequestFactory());
        
        RestSearchResultPage restSearchResultPage =
                restTemplate.getForObject(codingConceptURL, RestSearchResultPage.class, params);
        
        if (restSearchResultPage != null && restSearchResultPage.getResults() != null
        		&& restSearchResultPage.getResults().length > 0 
        		&& restSearchResultPage.getResults()[0].getReferencedConcept() != null){
          
        	    UUID uuid =	restSearchResultPage.getResults()[0].getReferencedConcept().getIdentifiers().getUuids().get(0); 
        	    params.clear();
        	    params.put("UUID", String.valueOf(uuid.toString()));
        	    
        	    RestDynamicSememeVersion restDynamicSememeVersion[] =
            		restTemplate.getForObject(codingComponentURL, RestDynamicSememeVersion[].class, params);
        	    
        	    if (restDynamicSememeVersion != null && restDynamicSememeVersion.length > 0 
            		&& restDynamicSememeVersion[0].getDataColumns() != null
            		&& !restDynamicSememeVersion[0].getDataColumns().isEmpty()){
        	        String object = (String)restDynamicSememeVersion[0].getDataColumns().get(0).getData().toString();
        	        return object;
            }            
        }
        
      }
     return null;
    }

    
    
    
    //**** Integration Test method, not for Production ****//
    public RestSearchResultPage getTestRxcuid(String vuid) {   
    	
    	String searchPPSNVUID = "https://DNS.DNS   :PORT/isaac-rest_2/rest/1/search/sememes?query={vuid}&treatAsString=true&expand=referencedConcept";
    	Map<String,String> params = new HashMap<String, String>();
    	params.put("vuid", "4001164");
            	
        RestTemplate restTemplate = new RestTemplate();
        
        String restSearchResultPageString = restTemplate.getForObject(searchPPSNVUID, String.class, params);
        RestSearchResultPage restSearchResultPage =
                restTemplate.getForObject(searchPPSNVUID, RestSearchResultPage.class, params);
        
        System.out.println(restSearchResultPageString);
        Set<String> uuids = new HashSet<String>();
        Set<Integer> matchIds = new HashSet<Integer>();
        for (int i=0; i < restSearchResultPage.getResults().length; i++){
        	params.clear();
        	RestSearchResult result = restSearchResultPage.getResults()[i];
//        	Integer matchId = result.getMatchNid();
//        	System.out.println("RestSearchResult");
//        	System.out.println("Concept Result: "  + result.getMatchNid());
//        	System.out.println("Concept Identifiers Type: "  + result.getReferencedConcept().getIdentifiers().getType().getFriendlyName());
//        	System.out.println("Concept Identifiers Nid: "  + result.getReferencedConcept().getIdentifiers().getNid());
//        	System.out.println("Concept Identifiers Class: "  + result.getReferencedConcept().getIdentifiers().getClass());
//        	System.out.println("Concept Identifiers UUIDS: "  + result.getReferencedConcept().getIdentifiers().getUuids().get(0).toString());
        	matchIds.add(result.getMatchNid());
        	for (UUID uuid: result.getReferencedConcept().getIdentifiers().getUuids()){
        		uuids.add(uuid.toString());
        	}
        }
  //      System.out.println("UUIDS List size: " + uuids.size() );
        for (String uuid : uuids){
        	System.out.println("----------------------");
            Map<String,String> params2 = new HashMap<String, String>();
        	params2.put("uuid", uuid);
        	String searchConcept = "https://DNS.DNS   :PORT/isaac-rest_2/rest/1/concept/chronology/{uuid}";
        	//RestConceptChronology restConceptChronology = restTemplate.getForObject(searchConcept, RestConceptChronology.class, params2);
        	String restConceptChronologyString = restTemplate.getForObject(searchConcept, String.class, params2);
 //       	System.out.println(restConceptChronologyString);
            
        }

  //      System.out.println("******************************");
        
        for (Integer matchId : matchIds){
//        	System.out.println("Rest Sememe Version");
        	params.clear();
        	params.put("matchId", matchId.toString());
        	String searchVersionString = "https://DNS.DNS   :PORT/isaac-rest_2/rest/1/sememe/version/{matchId}?expand=chronology";
        	String restSememeTypeString = restTemplate.getForObject(searchVersionString, String.class, params);
//        	RestSememeVersion  restSememeVersion  = restTemplate.getForObject(searchVersionString, RestSememeVersion.class);
   //     	System.out.println(restSememeTypeString);

        }
        
        return restSearchResultPage;
    }


  public String getCodingConceptURL() {
    return codingConceptURL;
  }



  public void setCodingConceptURL(String codingConceptURL) {
    this.codingConceptURL = codingConceptURL;
  }



  public String getCodingComponentURL() {
    return codingComponentURL;
  }



  public void setCodingComponentURL(String codingComponentURL) {
    this.codingComponentURL = codingComponentURL;
  }

}
