/**
 * Source file created in 2011 by Southwest Research Institute
 * 
 * 
 */


package gov.va.med.pharmacy.peps.domain.common.utility.converter;


import java.util.ArrayList;
import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.CopayTierAssocVo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProdCopayTierAssocDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;


/**
 * Convert to/from {@link DrugUnitVo} and {@link EplDrugUnitDo}.
 */
public class CopayTierAssociationConverter extends
        AssociationConverter<CopayTierAssocVo, EplProdCopayTierAssocDo, EplProductDo> {

    /**
     * Converts between EplCopayTierDo and CopayTierVo objects and vice versa.
     */
    private CopayTierConverter copayTierConverter;

    
    public final CopayTierConverter getCopayTierConverter() {
        return copayTierConverter;
    }

    
    public final void setCopayTierConverter(CopayTierConverter copayTierConverter) {
        this.copayTierConverter = copayTierConverter;
    }

    /**
     * Converts an {@link EplProdCopayTierAssocDo} to a {@link CopayTierAssocVo}. Does <em>not</em> set the product
     * property of the returned object.
     * @param data ProductCopayTierDo
     * @return ProductCopayTierVo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject(
     *      gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    @Override
    protected final CopayTierAssocVo toValueObject(EplProdCopayTierAssocDo data) {
        CopayTierAssocVo value = new CopayTierAssocVo();
        value.setEplId(data.getId());
        value.setCopayTier(copayTierConverter.convert(data.getEplCopayTier()));
        value.setEffectiveDate(data.getEffectiveDate());
        value.setEndDate(data.getEndDate());
        
        return value;
    }

    /**
     * Fully copies the ProductCopayTierVo to a EplProductCopayTierDo.
     * 
     * @param data EplProductCopayTierDo
     * @return fully populated EplProductCopayTierDo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject
     *      (gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    @Override
    protected final EplProdCopayTierAssocDo toDataObject(CopayTierAssocVo data) {
        EplProdCopayTierAssocDo value = new EplProdCopayTierAssocDo();          
        value.setId(data.getEplId());
        value.setEplCopayTier(copayTierConverter.convert(data.getCopayTier()));
        value.setEffectiveDate(data.getEffectiveDate());
        value.setEndDate(data.getEndDate());        
        return value;
    }
   
    /**
     * Fully copies the ProductCopayTierVo to a EplProductCopayTierDo.
     * 
     * @param dataList EplProductCopayTierDo
     * @return fully populated EplProductCopayTierDo List
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject
     *      (gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    protected final List<EplProdCopayTierAssocDo> toDataObject(List<CopayTierAssocVo> dataList) {
        List<EplProdCopayTierAssocDo> list = new ArrayList<EplProdCopayTierAssocDo>();
        
        for (CopayTierAssocVo data : dataList) {
            EplProdCopayTierAssocDo value = new EplProdCopayTierAssocDo();
            value.setId(data.getEplId());
            value.setEplCopayTier(copayTierConverter.convert(data.getCopayTier()));
            value.setEffectiveDate(data.getEffectiveDate());
            value.setEndDate(data.getEndDate());
            list.add(value);
        }
        
        return list;
    }
   

    /**
     * Minimally value object converter for ProductCopayTierVo History.
     * @param data EplProductCopayTierDo
     * @return ProductCopayTierVo
     */
    @Override
    protected final CopayTierAssocVo toMinimalValueObject(
            EplProdCopayTierAssocDo data) {
        CopayTierAssocVo dataObject = toValueObject(data);
        return dataObject;
    }

    @Override
    protected final EplProdCopayTierAssocDo toDataObject(CopayTierAssocVo data, 
            EplProductDo parent, int sequence) {
        EplProdCopayTierAssocDo productCopayTier = convert(data);    
        return productCopayTier;
    }
}
