
package gov.va.med.pharmacy.peps.domain.common.utility.converter;


import java.util.ArrayList;
import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.CopayTierVo;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.domain.common.model.EplCopayTierDo;


/**
 * Convert to/from {@link CopayTierVo} and {@link EplCopayTierDo}.
 */
public class CopayTierConverter extends
        Converter<CopayTierVo, EplCopayTierDo> {

    /**
     * to CopayTierVo from CopayTierDo.     
     * @param data EplCopayTierDo
     * @return CopayTierVo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject(
     *      gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    @Override
    protected CopayTierVo toValueObject(EplCopayTierDo data) {
        CopayTierVo value = new CopayTierVo();
                
        value.setId(data.getEplId().toString());
        value.setTierLevel(data.getTierLevel());
        value.setDescription(data.getDescription());
        
        value.setItemStatus(ItemStatus.valueOf(data.getItemStatus()));
        value.setRevisionNumber(data.getRevisionNumber());
        value.setRequestItemStatus(RequestItemStatus.APPROVED);
        value.setCreatedBy(data.getCreatedBy());
        value.setCreatedDate(data.getCreatedDtm());
        value.setModifiedBy(data.getLastModifiedBy());
        value.setModifiedDate(data.getLastModifiedDtm());
        if (data.getInactivationDate() != null) {
            value.setInactivationDate(data.getInactivationDate());
        }
        
        return value;
    }
    
    /**
     * to CopayTierVo from EplCopayTierDo.     
     * @param dataList EplCopayTierDo
     * @return CopayTierVo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject(
     *      gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    protected List<CopayTierVo> toValueObject(List<EplCopayTierDo> dataList) {
        List<CopayTierVo> list = new  ArrayList<CopayTierVo>();
        
        for (EplCopayTierDo data : dataList) {
            list.add(toValueObject(data));
        }
        
        return list;
    }

    /**
     * Fully copies the CopayTierVo to a EplCopayTierDo.
     * 
     * @param value EplCopayTierDo
     * @return fully populated EplCopayTierDo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject
     *      (gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    @Override
    protected EplCopayTierDo toDataObject(CopayTierVo value) {
        EplCopayTierDo data = new EplCopayTierDo();
        
       
        data.setEplId(Long.valueOf(value.getId()));

        data.setTierLevel(value.getTierLevel());
        data.setDescription(value.getDescription());
        data.setRevisionNumber(value.getRevisionNumber());
        data.setCreatedBy(value.getCreatedBy());
        data.setCreatedDtm(value.getCreatedDate());
        data.setLastModifiedBy(value.getModifiedBy());
        data.setLastModifiedDtm(value.getModifiedDate());

        if (value.getItemStatus() != null) {
            data.setItemStatus(value.getItemStatus().name());
        }     
        
        if (value.getInactivationDate() != null) {
            data.setInactivationDate(value.getInactivationDate());
        }

        
        return data;
    }
   
    /**
     * Fully copies the CopayTierVo to a EplCopayTierDo.
     * 
     * @param dataList EplCopayTierDo
     * @return fully populated EplCopayTierDo List
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject
     *      (gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    protected List<EplCopayTierDo> toDataObject(List<CopayTierVo> dataList) {
        List<EplCopayTierDo> list = new ArrayList<EplCopayTierDo>();
        
        for (CopayTierVo data : dataList) {           
            list.add(toDataObject(data));
        }
        
        return list;
    }
   

    /**
     * Minimally value object converter for CopayTierVo History.
     * @param data EplProductCopayTierDo
     * @return ProductCopayTierVo
     */
    @Override
    protected CopayTierVo toMinimalValueObject(
            EplCopayTierDo data) {
        CopayTierVo dataObject = new CopayTierVo();
        dataObject = toValueObject(data);

        return dataObject;
    }
}
