

package gov.va.med.pharmacy.peps.domain.common.utility.converter;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import gov.va.med.pharmacy.peps.common.object.VistaStatus;
import gov.va.med.pharmacy.peps.common.vo.EplVistaUpdateVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplVistaUpdateStatusDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfUpdateFileDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplVistaStatusDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplVistaUpdateDo;

@Component
public class EplVistaUpdateConverter extends Converter<EplVistaUpdateVo, EplVistaUpdateDo> {
  
  private final EplVistaUpdateStatusDao vistaUpdateStatusDao;
  
  /**
   * Constructs an instance with the required properties.
   * @param pVistaUpdateStatusDao the {@link EplVistaUpdateStatusDao} to use.
   */
  @Autowired
  public EplVistaUpdateConverter(EplVistaUpdateStatusDao pVistaUpdateStatusDao) {
    vistaUpdateStatusDao = pVistaUpdateStatusDao;
  }

    @Override
    protected EplVistaUpdateDo toDataObject(EplVistaUpdateVo data) {
        EplVistaUpdateDo updateDo = new EplVistaUpdateDo();

        EplNdfUpdateFileDo ndfUpdateFileDo = new EplNdfUpdateFileDo();
        ndfUpdateFileDo.setFileName(data.getFileName());
        
        updateDo.setComments(data.getComments());
        updateDo.setEplVistaStatus(retrieveStatus(data.getStatus()));
        updateDo.setSite(data.getSite());
        updateDo.setEplNdfUpdateFileDo(ndfUpdateFileDo);
        updateDo.setTransmissionDate(data.getTransmissionDate());
        return updateDo;
    }
    
    /**
     * Retrieves the {@link EplVistaStatusDo} from the database by name.
     * @throws IllegalArgumentException if the given name does not represent a valid status
     */
    private EplVistaStatusDo retrieveStatus(final String name) {
        final VistaStatus vistaStatus = VistaStatus.valueOf(name); // valueOf() throws IllegalArgumentException
        return vistaUpdateStatusDao.getStatusByName(vistaStatus);
    }

    @Override
    protected EplVistaUpdateVo toValueObject(EplVistaUpdateDo data) {
        EplVistaUpdateVo eplVistaUpdateVo = new EplVistaUpdateVo();
        eplVistaUpdateVo.setComments(data.getComments());
        eplVistaUpdateVo.setFileName(data.getEplNdfUpdateFileDo().getFileName());
        eplVistaUpdateVo.setSite(data.getSite());
        eplVistaUpdateVo.setStatus(data.getEplVistaStatus().getStatusName());
        eplVistaUpdateVo.setTransmissionDate(data.getTransmissionDate());
        
        return eplVistaUpdateVo;
    }

}
