
package gov.va.med.pharmacy.peps.domain.common.utility.converter;


import java.util.ArrayList;
import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorVo;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.domain.common.model.EplFormularyDesignatorDo;


/**
 * Convert to/from {@link FormularyDesignatorVo} and {@link EplFormularyDesignatorDo}.
 */
public class FormularyDesignatorConverter extends
        Converter<FormularyDesignatorVo, EplFormularyDesignatorDo> {

    /**
     * To FormularyDesignatorVo from FormularyDesignatorDo. 
     *     
     * @param data EplFormularyDesignatorDo
     * @return FormularyDesignatorVo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject(
     *      gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    @Override
    protected FormularyDesignatorVo toValueObject(EplFormularyDesignatorDo data) {
        FormularyDesignatorVo value = new FormularyDesignatorVo();
        value.setId(data.getEplId().toString());
        value.setName(data.getName());
        value.setDescription(data.getDescription());
        value.setItemStatus(ItemStatus.valueOf(data.getItemStatus()));
        value.setRevisionNumber(data.getRevisionNumber());
        value.setRequestItemStatus(RequestItemStatus.APPROVED);
        value.setCreatedBy(data.getCreatedBy());
        value.setCreatedDate(data.getCreatedDtm());
        value.setModifiedBy(data.getLastModifiedBy());
        value.setModifiedDate(data.getLastModifiedDtm());
        value.setInactivationDate(data.getInactivationDate());
        return value;
    }
    
    /**
     * to FormularyDesignatorVo from EplFormularyDesignatorDo.     
     * @param dataList EplFormularyDesignatorDo
     * @return FormularyDesignatorVo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject(
     *      gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    protected List<FormularyDesignatorVo> toValueObject(List<EplFormularyDesignatorDo> dataList) {
        List<FormularyDesignatorVo> list = new  ArrayList<FormularyDesignatorVo>();
        
        for (EplFormularyDesignatorDo data : dataList) {
            list.add(toValueObject(data));
        }
        
        return list;
    }

    /**
     * Fully copies the FormularyDesignatorVo to a EplFormularyDesignatorDo.
     * 
     * @param value EplFormularyDesignatorDo
     * @return fully populated EplFormularyDesignatorDo
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject
     *      (gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    @Override
    protected EplFormularyDesignatorDo toDataObject(FormularyDesignatorVo value) {
        EplFormularyDesignatorDo data = new EplFormularyDesignatorDo();
        data.setEplId(Long.valueOf(value.getId()));
        data.setName(value.getName());
        data.setDescription(value.getDescription());
        data.setRevisionNumber(value.getRevisionNumber());
        data.setCreatedBy(value.getCreatedBy());
        data.setCreatedDtm(value.getCreatedDate());
        data.setLastModifiedBy(value.getModifiedBy());
        data.setLastModifiedDtm(value.getModifiedDate());
        data.setItemStatus(value.getItemStatus().name());
        data.setInactivationDate(value.getInactivationDate());
        return data;
    }
   
    /**
     * Fully copies the FormularyDesignatorVo to a EplFormularyDesignatorDo.
     * 
     * @param dataList EplFormularyDesignatorDo
     * @return fully populated EplFormularyDesignatorDo List
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter#toDataObject
     *      (gov.va.med.pharmacy.peps.common.vo.ValueObject)
     */
    protected List<EplFormularyDesignatorDo> toDataObject(List<FormularyDesignatorVo> dataList) {
        List<EplFormularyDesignatorDo> list = new ArrayList<EplFormularyDesignatorDo>();
        
        for (FormularyDesignatorVo data : dataList) {           
            list.add(toDataObject(data));
        }
        
        return list;
    }
   

    /**
     * Minimally value object converter for FormularyDesignatorVo History.
     * @param data EplFormularyDesignatorDo
     * @return FormularyDesignatorVo
     */
    @Override
    protected FormularyDesignatorVo toMinimalValueObject(
            EplFormularyDesignatorDo data) {
        FormularyDesignatorVo dataObject = new FormularyDesignatorVo();
        dataObject = toValueObject(data);
        return dataObject;
    }
}
