package gov.va.med.pharmacy.peps.service.common.capability;

import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;


public interface NdfFileSyncCapability {
    
    /**
     * Accepts a new ManagedItem and saves the information for sending to VistA later.
     * 
     * @param item ManagedItem to send to VistA
     * @param user current UserVo
     * @throws ValidationException 
     */
    void saveNewItem(EplNdfOutgoingDifferences eplNdfOutgoingDifferences) 
        throws ValidationException;

    /**
     * Accepts a modified ManagedItem and saves the information for sending to VistA later.
     * 
     * @param item ManagedItem to send to VistA
     * @param differences Differences between the old and new value object
     * @param user current UserVo
     * @throws ValidationException 
     */
    void saveModifiedItem(EplNdfOutgoingDifferences eplNdfOutgoingDifferences) 
        throws ValidationException;

  /**
   * Removes all pending outgoing differences (that have not been associated to a file yet) for the specified file, multiple
   * field number and vistaIen. All fields with the VistA field number base would be included in the delete. Ex. For Copay Tier,
   * the caller would specify 45. Fields [45,.01], [45,1], and [45,2] would be deleted for the vistaIen specified.
   * 
   * The VistA IEN specified should be the main entity's ien. For example, a Copay Tier is associated to a product. THe Vista
   * IEN on the outgoing differences file is a comma-separated aggregation of the product's IEN and the pseudo IEN of the Copay
   * Tier (Ex. [123456,1], [123456,2],...). The caller should specify just the 123456 in this case.
   * 
   * @param vistaFileNumber
   *          VistA File Number
   * @param vistaFieldNumberBase
   *          Base of the VistA Field Number of the multiple field. Ex. Copay Tier would be 45.
   * @param vistaIen
   *          VistA IEN to which the record is associated.
   */
	void removePendingOutgoingDifferencesForMultipleField(String vaProductFile, String fieldNumberBase,
			String vistaIen);

}
