package gov.va.med.pharmacy.peps.service.common.update;

import java.util.Collection;

import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.domain.common.capability.DrugSeverityDomainCapability;
import gov.va.med.pharmacy.peps.service.common.capability.NdfFileSyncCapability;

/**
 * NdfFileSyncProcessor is an interface for classes that generate outgoing difference records for the NDF Update File.
 */
public interface NdfFileSyncProcessor {
    
    /**
     * Process a new ManagedItemVo.
     *
     * @param managedItem the new managed item
     * @param user the user who created the new item
     */
    void processNew(ManagedItemVo managedItem, UserVo user);
       
    /**
     * Process a modified ManagedItemVo.
     *
     * @param managedItem the modified managed item. This reference should be the updated version.
     * @param user the user who made the modification
     * @param differences the differences between the previous version and the modified version of the ManagedItemVo
     */
    void processModified(ManagedItemVo managedItem, UserVo user, Collection<Difference> differences );     
        
}
