package gov.va.med.pharmacy.peps.service.common.update;

/**
 * SyncProcessorElement is a data container for the NdfFileSyncProcessor classes to use as they are processing fields
 * and gathering the data needed to eventually create an EplNdfOutgoingDifference record.  
 */
public class SyncProcessorElement {

  /** The vista ien. */
  private String vistaIen; 
  
  /** The new value. */
  private String newValue; 
  
  /** The user name. */
  private String userName;
  
  /** The field id. */
  private String fieldId;
  
  /** The old value. */
  private String oldValue;
  
  /**
   * Instantiates a new sync processor element with the parameters used in the various insert methods in the SyncProcessors.
   *
   * @param vistaIen the vista ien to the item with which the field is associated
   * @param newValue the new value of the field
   * @param userName the user name making or approving the change
   * @param fieldId the Vista field number
   * @param oldValue the old value used only when modifying a field
   */
  public SyncProcessorElement(String vistaIen, String newValue, String userName, String fieldId, String oldValue) {
    super();
    this.vistaIen = vistaIen;
    this.newValue = newValue;
    this.userName = userName;
    this.fieldId = fieldId;
    this.oldValue = oldValue;
  }
  
  /**
   * Instantiates a new sync processor element with the parameters used in the various insert methods in the SyncProcessors.
   *
   * @param vistaIen the vista ien
   * @param newValue the new value
   * @param userName the user name
   * @param fieldId the field id
   */
  public SyncProcessorElement(String vistaIen, String newValue, String userName, String fieldId) {
    super();
    this.vistaIen = vistaIen;
    this.newValue = newValue;
    this.userName = userName;
    this.fieldId = fieldId;
  }

  /**
   * Instantiates a new sync processor element with the parameters used in the various insert methods in the SyncProcessors.
   *
   * @param vistaIen the vista ien
   * @param newValue the new value
   * @param userName the user name
   */
  public SyncProcessorElement(String vistaIen, String newValue, String userName) {
    super();
    this.vistaIen = vistaIen;
    this.newValue = newValue;
    this.userName = userName;
  }

  /**
   * Gets the vista ien.
   *
   * @return the vistaIen
   */
  public final String getVistaIen() {
    return vistaIen;
  }
  
  /**
   * Sets the vista ien.
   *
   * @param vistaIen the vistaIen to set
   */
  public final void setVistaIen(String vistaIen) {
    this.vistaIen = vistaIen;
  }
  
  /**
   * Gets the new value.
   *
   * @return the newValue
   */
  public final String getNewValue() {
    return newValue;
  }
  
  /**
   * Sets the new value.
   *
   * @param newValue the newValue to set
   */
  public final void setNewValue(String newValue) {
    this.newValue = newValue;
  }
  
  /**
   * Gets the user name.
   *
   * @return the userName
   */
  public final String getUserName() {
    return userName;
  }
  
  /**
   * Sets the user name.
   *
   * @param userName the userName to set
   */
  public final void setUserName(String userName) {
    this.userName = userName;
  }
  
  /**
   * Gets the field id.
   *
   * @return the fieldId
   */
  public final String getFieldId() {
    return fieldId;
  }
  
  /**
   * Sets the field id.
   *
   * @param fieldId the fieldId to set
   */
  public final void setFieldId(String fieldId) {
    this.fieldId = fieldId;
  }
  
  /**
   * Gets the old value.
   *
   * @return the oldValue
   */
  public final String getOldValue() {
    return oldValue;
  }
  
  /**
   * Sets the old value.
   *
   * @param oldValue the oldValue to set
   */
  public final void setOldValue(String oldValue) {
    this.oldValue = oldValue;
  }
}
