package gov.va.med.pharmacy.peps.domain.common.dao;

//import static org.hamcrest.CoreMatchers.equalTo;
//import static org.hamcrest.CoreMatchers.not;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.assertThat;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.pharmacy.peps.domain.common.dao.EplProdCopayTierAssocDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplProdCopayTierAssocDo;
import gov.va.med.pharmacy.peps.domain.common.utility.test.AbstractDomainIT;
import org.junit.Assert;
import gov.va.med.pharmacy.peps.common.vo.ReportCopayTierChangeVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;

import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;

/**
 * ProductCopayTierTest
 */

@Component
public class EplProdCopayTierAssocIT extends AbstractDomainIT {
    
    @Autowired
    private EplProdCopayTierAssocDao eplProductCopayTierDaoImpl;
    
    @Test
    //@Rollback(false)
    @Transactional    
    public void testSave() {
     
        EplProdCopayTierAssocDo copayProductTier = new EplProdCopayTierAssocDo();
//        Long assocProduct = 5211459200459419L;
//        Long assocCopayTier = 99999L;

        copayProductTier.setId((long) 100000L);
//        copayProductTier.setProductFk(assocProduct);
//        copayProductTier.setCopayTierFk(assocCopayTier);        
        copayProductTier.setEffectiveDate(Calendar.getInstance().getTime());    
        copayProductTier.setEndDate(Calendar.getInstance().getTime());         
        copayProductTier.setCreatedBy("360");
        copayProductTier.setCreatedDtm(Calendar.getInstance().getTime());   
        eplProductCopayTierDaoImpl.save(copayProductTier);
        System.out.println(copayProductTier.getId());
        Assert.assertNotNull(copayProductTier.getId());
        
    }
    
    @Test
    //@Rollback(false)
    @Transactional    
    public void testRetrievePPSCopayChangeData() throws Exception {
        boolean doAssertions = false;
        boolean printResults = false;
        Date startDate = null;
        DateFormat df = new SimpleDateFormat("MM/dd/yyyy", Locale.US);
        startDate = df.parse("12/01/2016");          
        int includeActive = 1;
        int includeInactive = 1;
        String subFieldDelimiter = "|";
        List<ReportCopayTierChangeVo> results = eplProductCopayTierDaoImpl.retrieveProductsWithCopayChangeData(startDate, includeActive, 
            includeInactive, subFieldDelimiter);
        assertThat(results, is(notNullValue()));
        
        //Only do these assertions if the developer has looked at the current database and updated expected values.  Automated test should
        //not be dependent on database state, and therefore should skip these assertions.
        if (doAssertions){
          assertThat(results.size(), not(equalTo(0)));
          assertThat(results.size(), equalTo(90));
        }
        
        // The following can be used to print out all the records
        if (printResults){
          final String OUTPUT_DELIMITER = "^";        
          for (ReportCopayTierChangeVo reportCopayTierChangeVo : results) {          
            System.out.println(reportCopayTierChangeVo.getEplId() + OUTPUT_DELIMITER
                + reportCopayTierChangeVo.getVaProductName() + OUTPUT_DELIMITER
                + nullToEmpty(reportCopayTierChangeVo.getTier()) + OUTPUT_DELIMITER
                + formatDate(reportCopayTierChangeVo.getEffectiveDate()) + OUTPUT_DELIMITER
                + formatDate(reportCopayTierChangeVo.getEndDate()) + OUTPUT_DELIMITER
                + nullToEmpty(reportCopayTierChangeVo.getFutureTierLevel()) + OUTPUT_DELIMITER
                + formatDate(reportCopayTierChangeVo.getFutureEffectiveDate()) + OUTPUT_DELIMITER
                + nullToEmpty(reportCopayTierChangeVo.getDrugClass())
                );
          }
        }
    }

    private String nullToEmpty(Object obj){
      if (obj == null){
        return "";
      }
      return obj.toString();
    }

    private String formatDate(Date obj){
      if (obj == null){
        return "";
      }
      final SimpleDateFormat dateFormatter = new SimpleDateFormat("dd-MMM-yy");   
      return dateFormatter.format(obj);
    }
    
    /**
     * getTestUser
     * @return UserVo used during testing
     */
    protected UserVo getTestUser() {
        UserVo user = new UserGenerator().getNationalManagerOne();
        user.setFirstName("Domain Test Case First");
        user.setLastName("Domain Test Case Last");
        user.setLocation("Domain Test Case Location");
        user.setStationNumber("666");
        user.setUsername("domainTestCase");
        
        return user;
    }       
}
