package gov.va.med.pharmacy.peps.domain.common.utility.impl;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.junit.*;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;       
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import gov.va.med.pharmacy.peps.common.utility.impl.test.EnvironmentUtilityStub;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.impl.SeqNumDomainCapabilityImpl;
import gov.va.med.pharmacy.peps.domain.common.dao.impl.EplProductDaoImpl;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability;
import gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm.RxNormClient;
import gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm.test.RxNormClientIT;
import gov.va.med.pharmacy.peps.domain.common.utility.test.AbstractDomainIT;

public class RxNormCapabilityImplMultipleProductsIT extends AbstractDomainIT implements ApplicationContextAware{

  private ApplicationContext context;
  private UserVo user;
  
  @Autowired
  private PlatformTransactionManager transactionManager;
  
 // @Autowired
 // private EplProductDaoImpl eplProductDao;
  
  private RxNormCapability rxNormCapability;
  private RxNormClient rxnormClient;
  private EnvironmentUtilityStub environmentUtility;
  private static final int TIMEOUT = 200;
  private static final int LONG_TIMEOUT = 1200;
  
  
  @Override
  public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
      this.context = applicationContext;
      
  }
  
  @Before
  public void setup(){    
    user = new UserVo();
    user.setUsername("AutomatedTest");
    RxNormClientIT.trustSelfSignedSSL();
    
    rxNormCapability =  (RxNormCapability)context.getBean("rxNormCapability");
    rxnormClient =  (RxNormClient)context.getBean("rxNormClient");
    
    environmentUtility = new EnvironmentUtilityStub();
    environmentUtility.setNational();
    
    SeqNumDomainCapabilityImpl seqNumCapability =  (SeqNumDomainCapabilityImpl)context.getBean("seqNumDomainCapability");
    seqNumCapability.setEnvironmentUtility(environmentUtility);        
    
    rxNormCapability.setEnvironmentUtility(environmentUtility);
  }
  
  @Test
  public void testgetAllProductCodesFromIsaac(){
    List<EplProductDo> products = fetchRxNormUpdateProducts();
    System.out.println("Number of products: " + products.size());
    int nullCodes = 0;
    int notNullCodes = 0;
    String code;
    for (EplProductDo product: products){
      code = rxnormClient.getCode(product.getVuid());
      if (code != null){
        notNullCodes++;
        System.out.println("Vuid: " + product.getVuid() + ", Code: " + code);
      }
      else{
        nullCodes++;
      }
    }
    System.out.println("Not null codes: " + notNullCodes);
    System.out.println("Null codes: " + nullCodes);
  }
  
  
 // @Test
  public void mimicRxNormUpdateCapabilityProcess(){
    try {
      List<EplProductDo> products = fetchRxNormUpdateProducts();
      processProducts(user, products);
      Assert.assertTrue(true);
    }
    catch (Throwable ex){
      Assert.fail(ex.getMessage());
    }
  }
  
  @Transactional(noRollbackForClassName="java.lang.Exception")   
  private List<EplProductDo> fetchRxNormUpdateProducts() {
    DefaultTransactionDefinition def;
    TransactionStatus status;
    def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
    def.setTimeout(LONG_TIMEOUT);
    status = transactionManager.getTransaction(def);
    
    List<EplProductDo> products = rxNormCapability.fetchRxNormUpdateProducts();
    transactionManager.commit(status);
    
    return products;
  }
  
  private void processProducts(UserVo user, List<EplProductDo> products) {    
    //trustSelfSignedSSL();
    Date runDate = new Date();
    for (EplProductDo product : products) {
          
            processEachProduct(user, product, runDate);
            // Delay for 100 milliseconds so that we never send more than 10 per second
            try {
              TimeUnit.MILLISECONDS.sleep(100);
            } catch(InterruptedException ie) {
              //ignore
            }
        }
  }
  
  @Transactional      
  private void processEachProduct(UserVo user, EplProductDo productDo, Date runDate){
      
        DefaultTransactionDefinition def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRED);
        TransactionStatus status = transactionManager.getTransaction(def);        
        def.setTimeout(TIMEOUT);
        rxNormCapability.populateAndPersist(user, productDo, runDate);
        // Job completes normally
        transactionManager.commit(status);
  }
  
}
