/**
 * Source file created in 2012 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.external.common.drugdatavendor.outbound.capability;


import java.util.Date;
import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.FdbAutoAddVo;
import gov.va.med.pharmacy.peps.common.vo.FdbAutoUpdateVo;



/**
 * DrugReferenceAutoCapability encapsulates the FDB behavior for the auto update code.
 * It has a method for the newly added entries in FDB and a method for the newly updated entries.
 *
 */
public interface DrugReferenceAutoCapability {

  /**
   * The date pattern (e.g., for {@link java.text.SimpleDateFormat}) of the dates in the FDB_PACKAGEDDRUG table. Some
   * of the methods in this interface return dates in this format.
   */
  public String FDB_DATE_PATTERN = "yyyyMMdd";


    /**
     * Returns items added to the FDB database since the given date.
     * @param date the inclusive lower bound of the added date
     * @return a list of added items in arbitrary order
     */
    List<FdbAutoAddVo> getFdbAddedItems(Date date);


    /**
     * Returns items updated in the FDB database since the given date.
     * @param startDate the inclusive lower bound of the updated date
     * @return a list of updated items in arbitrary order
     */
    List<FdbAutoUpdateVo> getFdbUpdatedItems(Date startDate);
    
    
    /**
     * Generates a String containing the FDB version and issue date.
     * @return the String
     */
    String getFdbUpdateDate();
    
}
