/**
 * Source file created in 2006 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.external.common.sts.outbound.capability.impl;


import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import gov.va.med.pharmacy.peps.common.utility.ConfigFileUtility;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.StandardMedRouteVo;
import gov.va.med.pharmacy.peps.external.common.sts.outbound.capability.StsInterfaceCapability;

import ct.webservice.DNS.URL       .CtService;
import ct.webservice.DNS.URL       .ValueSetContentsListTransfer;
import ct.webservice.DNS.URL       .ValueSetContentsTransfer;

/**
 * STS Web Service Capability to perform operations on standard managed domain items.
 */
public class StsInterfaceCapabilityImpl implements StsInterfaceCapability {
    private static final org.apache.logging.log4j.Logger
      LOG = org.apache.logging.log4j.LogManager.getLogger(StsInterfaceCapabilityImpl.class);
    
    private static final Long VUID_LIST_NUM = 4708487L; // this is the VUID for the standard med route list 
    private static final String WSDL_MARKER = "?wsdl";

    /**
     * The default constructor loads the properties for the location of the STS service and instantiates the sts client.
     */
    public StsInterfaceCapabilityImpl() {
        LOG.debug("StsInterfaceCapabilityImpl constructor");
    }

    /**
     * getStsData.
     * @return  List<StandardMedicationRouteVo> dataList
     */
    public List<StandardMedRouteVo> getStsData() {

        List<StandardMedRouteVo> dataList = new ArrayList<StandardMedRouteVo>();

        try {
            LOG.debug("Before service instantiation");
            
            ConfigFileUtility configFile = new ConfigFileUtility();
            URL wsdlUrl = new URL(String.valueOf(configFile.getStsWebServiceURL())+WSDL_MARKER);
            
            CtService service = new CtService(wsdlUrl);
            LOG.debug("Retrieving the port from the following service: " + service); 
          
            ct.webservice.sts.URL       .Sts sts = service.getStsPort();
            Long l = new Long(VUID_LIST_NUM); 
    
            ValueSetContentsListTransfer response = sts.listValueSetContents(l, "current", null, null, null, null);
            LOG.debug("Total records is " + response.getTotalNumberOfRecords());
            List<ValueSetContentsTransfer> contents = response.getValueSetContents();
            
            for (ValueSetContentsTransfer data : contents) {
                LOG.debug("Name is " + data.getName());
                LOG.debug("VUID is " + data.getVUID());
                LOG.debug("Status is " + data.getStatus());
                StandardMedRouteVo vo = new StandardMedRouteVo();

                vo.setVuid(String.valueOf(data.getVUID()));
                vo.setValue(data.getName());

                if (data.getStatus().equals("active")) {
                    vo.setItemStatus(ItemStatus.ACTIVE);
                } else {
                    vo.setItemStatus(ItemStatus.INACTIVE);
                }
                
                dataList.add(vo);
            }
            
        } catch (Exception e) {
            LOG.error("STSInterface Exception ", e);
        }

        return dataList;
    }
}
