package gov.va.med.pharmacy.peps.common.utility.security;

/**
 * Wraps an untrusted parameter to a log entry. The parameter will be passed through validation when calling {@link
 * #toString()}. This is useful for the varargs log methods like {@link org.apache.logging.log4j.Logger#debug(String,
 * Object...)}.
 */
public class UntrustedLogParam {
  
  private final Object param;
  
  /**
   * Constructs an instance.
   * @param pLogParam the log parameter
   */
  public UntrustedLogParam(final Object pLogParam) {
    param = pLogParam;
  }
  
  /**
   * Returns the result of calling {@link Object#toString()} on the parameter after passing it through validation. If
   * the configured parameter is null, returns the string "null".
   */
  @Override
  public String toString() {
    if (param == null) {
      return "null";
    } else {
      return ESAPIValidator.validateStringInput(param.toString(), ESAPIValidationType.LOG_FORGING);
    }
  }

}
