package gov.va.med.pharmacy.peps.common.utility.security;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.util.Optional;

import org.junit.Test;

/**
 * Unit tests for {@link UntrustedLogParam}.
 */
public class UntrustedLogParamTest {

  /**
   * Tests a string that should not be modified.
   */
  @Test
  public void testToString_Clean() {
    // This type of this param doesn't matter.
    final Object param = Optional.of("a string");
    final UntrustedLogParam objUnderTest = new UntrustedLogParam(param);
    
    /* Behavior & verification */
    
    final String result = objUnderTest.toString();
    assertThat(result, equalTo(ESAPIValidator.validateStringInput(param.toString(), ESAPIValidationType.LOG_FORGING)));
  }
  
  /**
   * Tests a string that should be modified.
   */
  @Test
  public void testToString_Newline() {
    final Object param = "String with\nnewline";
    final UntrustedLogParam objUnderTest = new UntrustedLogParam(param);
    
    /* Behavior & verification */

    final String result = objUnderTest.toString();
    assertThat(result, equalTo(ESAPIValidator.validateStringInput(param.toString(), ESAPIValidationType.LOG_FORGING)));
  }
  
  /**
   * Tests passing null as the log parameter.
   */
  @Test
  public void testToString_Null() {
    final UntrustedLogParam objUnderTest = new UntrustedLogParam(null);

    /* Behavior & verification */
    
    final String result = objUnderTest.toString();
    assertThat(result, equalTo("null"));
  }

}
