package gov.va.med.pharmacy.peps.service.common.capability;

import java.io.IOException;
import java.util.Date;
import java.util.List;

import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdateProcessFile;

/**
 * Generates PPS-N reports included in the update file. Also generates the preview report.
 */
public interface MessageDataCapability {

  /**
   * Writes the "MESSAGE" section of the update file. This section is the "DATA UPDATE FOR NDF" report.
   * 
   * @param startDTM the start time of the update file generation process
   * @param ndfUpdateFile the file to write to
   * @throws IOException
   */
    public void messageReportData(Date startDTM, NdfUpdateProcessFile ndfUpdateFile)throws IOException;
    
    /**
     * Writes the "MESSAGE2" section of the update file. This section is the "UPDATED INTERACTIONS AND FDA MED GUIDE"
     * report.
     * 
     * @param startDTM the start time of the update file generation process
     * @param ndfUpdateFile the file to write to
     * @throws IOException
     */
    public void message2ReportData(Date startDTM, NdfUpdateProcessFile ndfUpdateFile)throws IOException;

    /**
     * Gets message data containing changes made for the update file preview report.
     * @param startDTM
     * @return List<String> containing message.
     */
    public List<String>  generateUpdateFilePreviewReport(Date startDTM);
    
    
}
