package gov.va.med.pharmacy.peps.service.common.capability;

import java.io.IOException;
import java.util.Date;
import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdateProcessFile;

/**
 * NdfUpdateProcessCapability contains the logic for putting the NDFUpdate Data into an update file and updating references 
 * in PPS_N to tie that data to the update file.
 * 
 */
public interface NdfUpdateProcessCapability {

    /**
     * Writes the data to the given file, closes it, and associates the data to the given NdfUpdateFileVo.
     *
     * @param newFileVo the new NdfUpdateFileVo
     * @param updateFile the update file to write to
     * @throws IOException Signals that an I/O exception has occurred.
     */
    public void addDataToFile(NdfUpdateFileVo newFileVo, NdfUpdateProcessFile updateFile) throws IOException;

    /**
     * Remove the file associations with all other data.
     *
     * @param fileSeqNo the file seq no to remove
     */
    public void removeAllFileAssociations(Long fileSeqNo);    
    
    /**
     * Returns Update File message data for the preview reports.
     * @param startDTM Date of the earliest data to include in the file
     * @return List<String> List containing message data
     */
    public List<String> getPreviewUpdateFileReportData(Date startDTM);
}
