package gov.va.med.pharmacy.peps.service.common.scheduler.tasks;

import gov.va.med.pharmacy.peps.service.common.capability.NdfUpdateCapability;
import gov.va.med.pharmacy.peps.service.common.scheduler.ProcessStatus;
import gov.va.med.pharmacy.peps.service.common.scheduler.ProcessStatusUpdater;
import gov.va.med.pharmacy.peps.service.common.scheduler.jobs.NdfUpdateJob;
import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.common.vo.UserVo;

import org.apache.logging.log4j.Logger; 
import org.springframework.beans.factory.annotation.Autowired;


/**
 * <p>Scheduled task for NDF Update File Creation.</p>
 * 
 * <p>Why is this class separate from {@link NdfUpdateJob}? It may be to make dependency injection easier since a new
 * job instance is created every time the job runs.</p>
 */
public class NdfUpdateTaskImpl implements NdfUpdateTask {
    
    private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(NdfUpdateTaskImpl.class);

  /**
   * The national setting for the job run state.
   */
  static final NationalSetting RUN_STATE_SETTING = NationalSetting.NDF_UPDATE_RUN_STATE;

  /**
   * The national setting for the job last run time.
   */
  static final NationalSetting LAST_RUN_SETTING = NationalSetting.NDF_UPDATE_LAST_RUN;

  private static final Long USER_ID = 42L;
  private static final String STATION = "999";
    
  private final NdfUpdateCapability ndfUpdateCapability;
    
  private final ProcessStatusUpdater processStatusUpdater;

  @Autowired
  public NdfUpdateTaskImpl(
      final NdfUpdateCapability pNdfUpdateCapability, final ProcessStatusUpdater pProcessStatusUpdater) {
    ndfUpdateCapability = pNdfUpdateCapability;
    processStatusUpdater = pProcessStatusUpdater;
  }

    @Override
    public void runNdfUpdateProcess() {
      
      final UserVo user = getBatchUser();

      ProcessStatus processStatus = ProcessStatus.RUNNING;
      processStatusUpdater.updateStatus(RUN_STATE_SETTING, processStatus, user);
        
      try {
        ndfUpdateCapability.createScheduledFile(user);
        processStatus = ProcessStatus.COMPLETED;
      } catch (Exception e) {
        LOG.error("Update file process failed with an Exception.", e);
        processStatus = ProcessStatus.ERROR;
      }

      processStatusUpdater.updateStatusAndRuntime(RUN_STATE_SETTING, processStatus, LAST_RUN_SETTING, user);
    }

    /**
     * Get the user to be used by the scheduled job.
     * @return UserVo
     */
    protected UserVo getBatchUser() {

        UserVo user = new UserVo();
        user.setFirstName("NDFUpdate");
        user.setLastName("UpdateFileProcess");
        user.setStationNumber(STATION);
        user.setUsername("NdfUpdateProcess");
        user.setLocation("NationalSite");
        user.setId(USER_ID);
        user.addRole(Role.PSS_PPSN_MANAGER );

        return user;

    }    
}
