

package gov.va.med.pharmacy.peps.service.common.capability.impl;

import org.hamcrest.Description;
import org.hamcrest.TypeSafeMatcher;


/**
 * Regular Expression Matcher for use in JUnit assertions.
 * 
 * Ex:
 * 
 * assertThat(actualString, RegExMatcher.matchesRegex(regexString));
 */
public class RegExMatcher extends TypeSafeMatcher<String> {

    /** The regex. */
    private final String regex;
    
    /**
     * Instantiates a new reg ex matcher.
     *
     * @param regex the regex
     */
    public RegExMatcher(String regex) {
        super();
        this.regex = regex;
    }

    /* (non-Javadoc)
     * @see org.junit.internal.matchers.TypeSafeMatcher#matchesSafely(java.lang.Object)
     */
    @Override
    public boolean matchesSafely(final String string) {
        return string.matches(regex);
    }

    /**
     * Matches regex.
     *
     * @param regex the regex
     * @return the reg ex matcher
     */
    // matcher method you can call on this matcher class
    public static RegExMatcher matchesRegex(final String regex) {
        return new RegExMatcher(regex);
    }

    /* (non-Javadoc)
     * @see org.hamcrest.SelfDescribing#describeTo(org.hamcrest.Description)
     */
    @Override
    public void describeTo(Description description) {
        description.appendText("matches regular expression=`" + regex + "`");
    }
}
