package gov.va.med.pharmacy.peps.service.common.migration;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.List;

import org.hamcrest.CoreMatchers;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.vo.MigrationCopayTierVo;

/**
 * Unit tests for {@link CopayTierImportReader}.
 */
public class CopayTierImportReaderTest {
  
  private static String EMPTY_COPAY_TIER_TEST_PATH = "copay_tier_migration/empty copay tier test.xlsx";
  private static String TWO_ROW_COPAY_TIER_TEST_PATH = "copay_tier_migration/2 row copay tier test.xlsx";
  
  /**
   * Returns an {@link InputStream} for the specified classpath resource. It is the caller's responsibility to
   * close the stream.
   * @param path the path to the classpath resource
   */
  private InputStream getClasspathReader(final String path) {
    return getClass().getClassLoader().getResourceAsStream(path);
  }
  
  /**
   * Verifies return value of {@link CopayTierImportReader#countCopayTierImport(InputStream)}.
   */
  @Test
  public void testCountCopayTierImport() throws IOException {
    
    /* Setup */
    final CopayTierImportReader objUnderTest = new CopayTierImportReader();
    final int expectedRows = 2;

    /* Behavior & Verification */
    final InputStream inputStream = getClasspathReader(TWO_ROW_COPAY_TIER_TEST_PATH);
    assertThat(objUnderTest.countCopayTierImport(inputStream), CoreMatchers.equalTo(expectedRows));
  }

  /**
   * Verifies that {@link CopayTierImportReader#getCopayTierImportList(InputStream)} properly returns an empty list for
   * an empty import file.
   */
  @Test
  public void testGetCopayTierImportListEmpty() {
    
    /* Setup */
    final CopayTierImportReader objUnderTest = new CopayTierImportReader();

    /* Behavior */
    final InputStream inputStream = getClasspathReader(EMPTY_COPAY_TIER_TEST_PATH);
    final List<MigrationCopayTierVo> result = objUnderTest.getCopayTierImportList(inputStream);
    
    /* Verification */
    assertThat(result, CoreMatchers.equalTo(Collections.emptyList()));
  }

}
