package gov.va.med.pharmacy.peps.service.common.scheduler.tasks;

import static org.easymock.EasyMock.*;

import java.io.IOException;
import java.util.Date;

import org.easymock.EasyMockSupport;
import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.service.common.capability.NdfUpdateCapability;
import gov.va.med.pharmacy.peps.service.common.scheduler.ProcessStatus;
import gov.va.med.pharmacy.peps.service.common.scheduler.ProcessStatusUpdater;

/**
 * Unit tests for {@link NdfUpdateTaskImpl}.
 */
public class NdfUpdateTaskImplTest extends EasyMockSupport {
  
  /**
   * Mock instantiated in {@link #setup()} for convenience.
   */
  private ProcessStatusUpdater mockProcessStatusUpdater;

  /**
   * Mock instantiated in {@link #setup()} for convenience.
   */
  private NdfUpdateCapability mockNdfUpdateCapability;
  
  @Before
  public void setup() {
    mockProcessStatusUpdater = strictMock(ProcessStatusUpdater.class);
    mockNdfUpdateCapability = strictMock(NdfUpdateCapability.class);
  }

  @Test
  public void testRunNdfUpdateProcess_Success() throws IOException {
    /* Setup */
    final NdfUpdateTaskImpl objUnderTest = new NdfUpdateTaskImpl(mockNdfUpdateCapability, mockProcessStatusUpdater);
    
    // Expected method calls.
    final UserVo user = objUnderTest.getBatchUser();
    mockProcessStatusUpdater.updateStatus(
        NdfUpdateTaskImpl.RUN_STATE_SETTING, ProcessStatus.RUNNING, user);
    mockNdfUpdateCapability.createScheduledFile(user);
    expect(mockProcessStatusUpdater.updateStatusAndRuntime(
        NdfUpdateTaskImpl.RUN_STATE_SETTING, ProcessStatus.COMPLETED, NdfUpdateTaskImpl.LAST_RUN_SETTING, user))
            .andReturn(new Date());
    
    /* Behavior & Verification */
    
    replayAll();
    
    objUnderTest.runNdfUpdateProcess();
    
    verifyAll();
  }

  @Test
  public void testRunNdfUpdateProcess_Error() throws IOException {
    /* Setup */
    final NdfUpdateTaskImpl objUnderTest = new NdfUpdateTaskImpl(mockNdfUpdateCapability, mockProcessStatusUpdater);
    
    // Expected method calls.
    final UserVo user = objUnderTest.getBatchUser();
    mockProcessStatusUpdater.updateStatus(
        NdfUpdateTaskImpl.RUN_STATE_SETTING, ProcessStatus.RUNNING, user);
    mockNdfUpdateCapability.createScheduledFile(user);
    expectLastCall().andThrow(new IOException("Simluate IOException."));
    expect(mockProcessStatusUpdater.updateStatusAndRuntime(
        NdfUpdateTaskImpl.RUN_STATE_SETTING, ProcessStatus.ERROR, NdfUpdateTaskImpl.LAST_RUN_SETTING, user))
            .andReturn(new Date());
    
    /* Behavior & Verification */
    
    replayAll();
    
    objUnderTest.runNdfUpdateProcess();
    
    verifyAll();
  }

}
