/**
 * Source file created in 2011 by Southwest Research Institute
 */

package gov.va.med.pharmacy.peps.presentation.common.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import gov.va.med.pharmacy.peps.common.vo.DeprecatedRematchVo;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.UnmatchRematchVo;
import gov.va.med.pharmacy.peps.service.common.session.FDBUpdateProcessService;
import gov.va.med.pharmacy.peps.service.common.session.NDFUpdateFileService;
import gov.va.med.pharmacy.peps.service.common.session.RematchService;

/**
 * Old v2.0 Rematch controller. This class should be removed before completing v3.0.
 * 
 * @since 2.0
 */
@Controller("unmatchedProductsController")
public class UnmatchedProductsController extends AbstractSearchController {

    private static final Logger LOG = LogManager.getLogger(UnmatchedProductsController.class);
    private static final String MATCH_TYPE_MAP = "matchTypeMap";
    private static final String MATCH_VO = "matchVo";
    private static final String SELECTED_MATCH_TYPE = "selectedMatchType";
    private static final String MATCH_TYPE_DOT = "MatchType.";

    private static SortedMap<MatchReason, String> MATCH_MAP;
    private static String UNMATCHED_PRODUCTS_VIEW = "unmatched.product.search";
    private static String UNMATCHED_PRODUCTS_REMATCH = "unmatch.rematch";

    @Autowired
    private RematchService rematchService;
    @Autowired
    private NDFUpdateFileService ndfUpdateFileService;
    @Autowired
    private FDBUpdateProcessService fDBUpdateProcessService;

    private DeprecatedRematchVo matchVo = new DeprecatedRematchVo();
    private MatchReason selectedMatch;

    @InitBinder
    protected void initBinderDTO(WebDataBinder binder) {
        binder.setDisallowedFields("dtoVar04292015");
    }

    /**
     * Request mapping for unmatched products
     * 
     * @throws Exception
     *             Exception
     */
    @RequestMapping(value = "/unmatchedproductscriteria.go", method = { RequestMethod.GET })
    public String unmatchedProductsTab(@ModelAttribute(MATCH_VO) DeprecatedRematchVo mvo, @RequestParam(
            value = "isFirstRun", required = false, defaultValue = "true") String isFirstRun, Model model,
            HttpServletRequest request, HttpSession session, Locale locale) throws Exception {

        boolean hasRun = StringUtils.isEmpty(isFirstRun) ? true : Boolean.parseBoolean(isFirstRun);
        // Update rematch list if the last update file has been approved
        Long currentStatus = ndfUpdateFileService.retrieveCurrentStatus().getStatus().getStatusId();
        if (currentStatus >= 7) {
            if ( currentStatus == 8 ) {
                rematchService.removeObsoleteProducts();
            }
            rematchService.addUnmatchedProducts(getUser());
        }

        if (hasRun) {
            initialize(locale);
            mvo.setMatchType(MatchReason.INACTIVATION);

            model.addAttribute(MATCH_TYPE_MAP, MATCH_MAP);
            model.addAttribute(MATCH_VO, mvo);

            return UNMATCHED_PRODUCTS_VIEW;
        } else {

            this.matchVo.setMatchType(mvo.getMatchType());
            mvo.clearAll();

            model.addAttribute(SELECTED_MATCH_TYPE, selectedMatch);
            model.addAttribute(MATCH_TYPE_MAP, MATCH_MAP);
            model.addAttribute(MATCH_VO, mvo);

            return UNMATCHED_PRODUCTS_VIEW;
        }
    }

    @RequestMapping(value = "/searchUnmatched.go", method = { RequestMethod.GET })
    public String searchUnmatched(@ModelAttribute(MATCH_VO) DeprecatedRematchVo mvo, Model model, HttpServletRequest request,
            HttpSession session, HttpServletResponse pResponse, Locale locale) throws Exception {

        pageTrail.clearTrail();
        pageTrail.addPage("reports", "Product Rematch", true);

        List<RematchVo> searchResults = null;

        if (mvo.getMatchType() != null) {
            this.matchVo.setMatchType(mvo.getMatchType());
        } else {
            if (selectedMatch != null) {
                this.matchVo.setMatchType(selectedMatch);
                mvo.setMatchType(selectedMatch);
            }
        }

        if (mvo.getMatchType() != null) {
            // Retrieves a list of the products that need a rematch for the user-selected matchType
            searchResults = rematchService.retrieveUnmatched(mvo.getMatchType().getId());
        }

        List<UnmatchRematchVo> unmatchedSearch = new ArrayList<UnmatchRematchVo>();

        if (searchResults != null) {
            for (RematchVo searchResult : searchResults) {

                Long productId = Long.parseLong(searchResult.getFromProduct().getId());
                // Using iterator to get the first element of a set.
                Long rematchedproductId = Long.parseLong(searchResult.getMatchProducts().iterator().next().getId());

                UnmatchRematchVo unmatchVo = new UnmatchRematchVo();
                ProductVo product =
                        (ProductVo) fDBUpdateProcessService.retrieveManagedItem(productId.toString(), EntityType.PRODUCT);
                ProductVo rematchedProduct = null;
                rematchedProduct =
                        (ProductVo) fDBUpdateProcessService.retrieveManagedItem(rematchedproductId.toString(),
                                EntityType.PRODUCT);

                unmatchVo.setId(searchResult.getEplRematchId());

                unmatchVo.setProductId(productId);
                unmatchVo.setCmop(product.getCmopId());
                unmatchVo.setProductName(product.getVaProductName());
                unmatchVo.setIen(searchResult.getFromProduct().getNdfProductIen().toString());

                unmatchVo.setMatchedIen(searchResult.getMatchProducts().iterator().next().getNdfProductIen().toString());
                unmatchVo.setMatchedProductId(rematchedProduct != null ? rematchedproductId : null);
                unmatchVo.setMatchedProductName(rematchedProduct != null ? rematchedProduct.getVaProductName() : null);
                unmatchVo.setMatchedCmop(rematchedProduct != null ? rematchedProduct.getCmopId() : null);
                unmatchVo.setMatchedScope(searchResult.getIsAutomaticRematch() ? "Automatic" : "Suggest");

                unmatchedSearch.add(unmatchVo);
            }
        }

        this.matchVo.setUnmatchedList(unmatchedSearch);
        mvo.setUnmatchedList(unmatchedSearch);
        mvo.setMatchType(this.matchVo.getMatchType());

        model.addAttribute("unmatchedProductsList", mvo.getUnmatchedList());
        model.addAttribute(SELECTED_MATCH_TYPE, selectedMatch);
        model.addAttribute(MATCH_TYPE_MAP, MATCH_MAP);
        model.addAttribute(MATCH_VO, mvo);

        return UNMATCHED_PRODUCTS_VIEW;
    }
     
    /**
     * Rematch.
     *
     * @param mvo
     *            the DeprecatedRematchVo
     * @param id
     *            the id
     * @param pModel
     *            the model
     * @return the string
     */
    @Deprecated // Rematch functionality being replaced by new flow located in RematchController
    @RequestMapping(value = "/rematch.go", method = RequestMethod.GET)
    public String rematch(@ModelAttribute(MATCH_VO) DeprecatedRematchVo mvo, @RequestParam(value = "id", required = false) String id,
            Model pModel) {
        DeprecatedRematchVo addStateBean = (DeprecatedRematchVo) flowScope.get(MATCH_VO);
        List<UnmatchRematchVo> rematches = (List<UnmatchRematchVo>) flowScope.get("rematchList");
        List<UnmatchRematchVo> products = (List<UnmatchRematchVo>) flowScope.get("productMatches");

        if (addStateBean == null) {
            addStateBean = this.matchVo;

            flowScope.put(MATCH_VO, addStateBean);
        }

        Long selectedProductId = null;

        if (mvo.getMatchType() != null) {
            this.matchVo.setMatchType(mvo.getMatchType());
        } else {
            if (this.matchVo.getMatchType() != null) {
                this.matchVo.getMatchType();
            }
        }

        if (rematches == null || rematches.isEmpty()) {

            if (id != null) {
                products = new ArrayList<UnmatchRematchVo>();
                rematches = new ArrayList<UnmatchRematchVo>();

                List<UnmatchRematchVo> searchResults = this.matchVo.getUnmatchedList();
                if (searchResults != null && !searchResults.isEmpty()) {
                    for (UnmatchRematchVo searchResult : searchResults) {
                        if (searchResult.getId().equals(id)) {
                            selectedProductId = searchResult.getProductId();
                            products.add(searchResult);
                            if (this.matchVo.getMatchType().equals(MatchReason.OTHER_CHANGE)) {
                                UnmatchRematchVo possibleRematch = new UnmatchRematchVo();
                                possibleRematch.setMatchedProductId(searchResult.getProductId());
                                possibleRematch.setMatchedProductName(searchResult.getProductName());
                                possibleRematch.setMatchedCmop(searchResult.getCmop());
                                possibleRematch.setMatchedIen(searchResult.getIen());
                                rematches.add(possibleRematch);
                            }

                        }
                    }
                }

                List<UnmatchRematchVo> matchedValues = rematchService.findRematchProducts(selectedProductId);
                rematches.addAll(matchedValues);

                if (products != null && !products.isEmpty()) {
                    boolean containsProduct = false;
                    UnmatchRematchVo productMatch = products.get(0);
                    if (productMatch.getMatchedProductId() != null) {
                        if (rematches != null && !rematches.isEmpty()) {
                            for (UnmatchRematchVo existingMatch : rematches) {
                                if (existingMatch.getMatchedProductId().equals(productMatch.getMatchedProductId())) {
                                    containsProduct = true;
                                }
                            }
                        }

                        if (!containsProduct) {
                            rematches.add(productMatch);

                        }
                    }
                }

            }

        }

        this.matchVo.setProductList(products);
        this.matchVo.setRematchList(rematches);

        mvo.setProductList(products);
        mvo.setRematchList(rematches);
        mvo.setMatchType(this.matchVo.getMatchType());

        pModel.addAttribute(MATCH_TYPE_MAP, MATCH_MAP);
        pModel.addAttribute(MATCH_VO, mvo);
        pModel.addAttribute("nationalMgrOrSuperRole", isNationalManagerOrSupervisorRole());

        return UNMATCHED_PRODUCTS_REMATCH;
    }

    @RequestMapping(value = "/match.go", method = RequestMethod.GET)
    public String match(@ModelAttribute(MATCH_VO) DeprecatedRematchVo mvo, Model pModel) {

        selectedMatch = matchVo.getMatchType();

        String matchedProductIen = null;

        List<UnmatchRematchVo> rematchLists = this.matchVo.getRematchList();

        if (rematchLists != null && !rematchLists.isEmpty()) {
            if (mvo.getChosenMatch() != null) {
                for (UnmatchRematchVo matchProduct : rematchLists) {
                    if (matchProduct.getMatchedProductId().equals(mvo.getChosenMatch().getProductId())) {
                        matchedProductIen = matchProduct.getMatchedIen();
                    }
                }
                rematchService.updateRematch((Long.valueOf(mvo.getTableId())), matchedProductIen,
                        Boolean.valueOf(mvo.getChosenMatch().isAuto()));
            } else {
                rematchService.updateRematch((Long.valueOf(mvo.getTableId())), null, null);
            }
        }

        this.matchVo.getProductList().clear();
        this.matchVo.getRematchList().clear();
        this.matchVo.getUnmatchedList().clear();

        pModel.addAttribute(MATCH_TYPE_MAP, MATCH_MAP);
        pModel.addAttribute(MATCH_VO, mvo);
        pModel.addAttribute("nationalMgrOrSuperRole", isNationalManagerOrSupervisorRole());

        return REDIRECT + "/searchUnmatched.go";
    }

    @RequestMapping(value = "/cancelMatch.go", method = RequestMethod.GET)
    public String cancelMatch(@ModelAttribute(MATCH_VO) DeprecatedRematchVo mvo, @RequestParam(value = "id", required = false) String id,
            Model pModel) {

        selectedMatch = matchVo.getMatchType();

        this.matchVo.getProductList().clear();
        this.matchVo.getRematchList().clear();
        this.matchVo.getUnmatchedList().clear();

        return REDIRECT + "/searchUnmatched.go";
    }

    /**
     * @param locale
     *            locale
     */
    private void initialize(Locale locale) {
        pageTrail.clearTrail();
        pageTrail.addPage("unmatchedProductSearch", "Rematch", true);
        MATCH_MAP = new TreeMap<MatchReason, String>();

        for (MatchReason element : MatchReason.values()) {

            MATCH_MAP.put(element, getMessageSource().getMessage(MATCH_TYPE_DOT + element.name(), null, locale));
        }

    }

    // Getters and Setters

    /**
     * Set reportService
     * 
     * @param nreportsService
     *            Service to use
     */
    public void setReportsService(RematchService nrematchService) {
        rematchService = nrematchService;
    }

    /**
     * Get reportService
     * 
     * @return the reportsService
     */
    public RematchService getRematchService() {
        return rematchService;
    }

    /**
     * Set reportType
     * 
     * @param reportType
     *            the reportType to set
     */
    public void setMatchType(MatchReason matchType) {
        matchVo.setMatchType(matchType);
    }

    /**
     * Get reportType
     * 
     * @return the reportType
     */
    public MatchReason getMatchType() {
        return matchVo.getMatchType();
    }

    /**
     * Get reportVo
     * 
     * @return reportVo
     */
    public DeprecatedRematchVo getMatchTypeVo() {
        return matchVo;
    }

    /**
     * Set reportVo
     * 
     * @param reportVo
     *            reportVo
     */
    public void setMatchTypeVo(DeprecatedRematchVo matchVo) {
        this.matchVo = matchVo;
    }

}
