/**
 * Source file created in 2012 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.presentation.common.controller.item;

import gov.va.med.pharmacy.peps.common.exception.OptimisticLockingException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.NdcVo;
import gov.va.med.pharmacy.peps.common.vo.ProcessedItemVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.common.vo.SearchCriteriaVo;
import gov.va.med.pharmacy.peps.common.vo.printtemplate.DefaultPrintTemplateFactory;
import gov.va.med.pharmacy.peps.presentation.common.controller.AbstractSearchController;
import gov.va.med.pharmacy.peps.presentation.common.controller.ControllerConstants;
import gov.va.med.pharmacy.peps.presentation.common.controller.bean.EditManagedItemBean;
import gov.va.med.pharmacy.peps.presentation.common.controller.bean.MultipleSelectItemBean;
import gov.va.med.pharmacy.peps.presentation.common.spring.annotation.RoleNeeded;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;


/**
 * 
 * MoveChildrenController's brief summary
 * 
 * Details of MoveChildrenController's operations, special dependencies
 * or protocols developers shall know about when using the class.
 *
 */
@Controller
@RoleNeeded(roles = { Role.PSS_PPSN_MANAGER, Role.PSS_PPSN_SUPERVISOR })
public class MoveChildrenController extends AbstractSearchController {

    @InitBinder
    protected void initBinderDTO(WebDataBinder binder) {
        binder.setDisallowedFields("dtoVar04292015");
    }

    /**
     * Add information to the model. This method is executed before any request
     * handler method, to ensure model state is established and attributes are added
     * to the model in the correct order.
     * 
     * @param entityType entityType specified in the request
     * @param model Spring model object
     */
    @ModelAttribute()
    public final void populateModel(
        @RequestParam(value = ControllerConstants.REQ_PARAM_KEY_ENTITYTYPE,
            required = false,
            defaultValue = ControllerConstants.REQ_PARAM_DEF_VALUE_ENTITYTYPE) EntityType entityType,
        Model model) {
        model.addAttribute(ControllerConstants.SEARCH_CRITERIA_KEY, createSearchCriteria(entityType));
        model.addAttribute("itemStatusTypeMap", createItemStatusTypeMap());
        model.addAttribute("requestStatusTypeMap", createRequestStatusTypeMap());
    }

    /**
     * Method to handle search for a new parent.
     *
     * @param entityType EntityType
     * @param searchCriteria SearchCriteriaVo
     * @param isFirstRun boolean
     * @param model Model
     * @return String
     * @throws ValidationException exception
     */
    @RequestMapping(value = "searchNewParent.go", method = RequestMethod.GET)
    public String searchNewParent(
        @RequestParam(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @ModelAttribute(ControllerConstants.SEARCH_CRITERIA_KEY) SearchCriteriaVo searchCriteria,
        @RequestParam(value = "isFirstRun", required = false, defaultValue = "true") boolean isFirstRun,
        Model model)
        throws ValidationException {

        pageTrail.addPage("move" + entityType.getChild().toLowerCase() + "s",
            "Move " + StringUtils.capitalize(entityType.getChild().toLowerCase()) + "s", true);

        if (!isFirstRun) {

            prepareSearchCriteria(searchCriteria);
            model.addAttribute("items", getManagedItemService().search(searchCriteria));
            switch (entityType.getChild()) {

                case PRODUCT:
                    model.addAttribute("printTemplate", DefaultPrintTemplateFactory.moveProductSelectOrderableItemSearch());
                    break;

                case NDC:
                    model.addAttribute("printTemplate", DefaultPrintTemplateFactory.moveNDCSelectProductSearch());
                    break;

                default:
            }
        }

        if (EntityType.NDC.equals(entityType.getChild())) {
            MultipleSelectItemBean multiEditBean = (MultipleSelectItemBean) flowScope.get("multiEditBean");
            EditManagedItemBean editManageItemBean = flowScope.get(EditManagedItemBean.class);
            ManagedItemVo item = editManageItemBean.getItem();

            ProductVo product = (ProductVo) item;
            List<NdcVo> ndcs = product.getNdcs();

            ArrayList packageTypes = new ArrayList();

            for (String childId : multiEditBean.getItemIds()) {
                for (NdcVo productNdcs : ndcs) {
                    if (childId.equals(productNdcs.getId())) {
                        packageTypes.add(productNdcs.getPackageType());
                    }
                }
            }

            // Removes duplicate items in list
            HashSet hs = new HashSet();
            hs.addAll(packageTypes);
            packageTypes.clear();
            packageTypes.addAll(hs);

            model.addAttribute(ControllerConstants.ITEM_KEY, item);
            model.addAttribute("packageTypes", packageTypes);
        }

        model.addAttribute("cancelUrl", "/cancelSearchNewParent.go");
        model.addAttribute("addTitle", ": Moving " + StringUtils.capitalize(entityType.getChild().toLowerCase()) + "s");

        return entityType.getViewName() + ".search";
    }

    /**
     * Method to handle moving children from one parent to another.
     *
     * @param entityType EntityType
     * @param itemId String
     * @return String
     * @throws OptimisticLockingException exception
     * @throws ValidationException exception
     */
    @RequestMapping(value = "{" + ControllerConstants.ENTITY_TYPE + "}/{" + ControllerConstants.ITEM_ID
        + "}/moveChildren.go", method = RequestMethod.GET)
    public String moveChildren(@PathVariable(value = ControllerConstants.ENTITY_TYPE) EntityType entityType,
        @PathVariable(value = ControllerConstants.ITEM_ID) String itemId) throws OptimisticLockingException, ValidationException {
  
      List<String> childrenProcessingMessages = new ArrayList<String>();
  
      // Get the new parent object
      EditManagedItemBean editBean = flowScope.get(EditManagedItemBean.class);
      ManagedItemVo toItem = getManagedItemService().retrieve(itemId, editBean.getItem().getEntityType());
  
      MultipleSelectItemBean multiEditBean = (MultipleSelectItemBean) flowScope.get("multiEditBean");
      if (multiEditBean == null) {
        throw new ValidationException(ValidationException.NO_NDCS_SELECTED);
      }
      // Get common children entity type
      EntityType entitiType = multiEditBean.getItemEntityTypes()[0];
      
      // Get the children by using children ids from presentation layer selection
      List<ManagedItemVo> children = new ArrayList<ManagedItemVo>();
      for (String childId : multiEditBean.getItemIds()) {
        ManagedItemVo child = getManagedItemService().retrieve(childId, entitiType);
        // Get the parent from which the children are moved. 
        // Presentation layer has fully populated object graph, use that instead of fetching it from db.
        child.setParent(editBean.getItem());
        children.add(child);
      }
  
      // Update parent-child relationships, affecting the new and the old parents
      List<ProcessedItemVo> childrenMsgs = getManagedItemService().updateParentChildRelationships(children, toItem, getUser());
  
      // Provide all the messages generated by processing of children in one collection.
      // Pass them to the presentation layer
      for (ProcessedItemVo childMsg : childrenMsgs) {
        childrenProcessingMessages.addAll(childMsg.getWarnings().getLocalizedErrors(getLocale()));
      }
      flashScope.put(ControllerConstants.WARNINGS, childrenProcessingMessages);
  
      // Update the old parent
      editBean.setItem(getManagedItemService().retrieve(editBean.getItem().getId(), editBean.getItem().getEntityType()));
  
      return REDIRECT + pageTrail.goToPreviousFlowUrl();
    }
  
    /**
     * Method to handle cancel search for a new parent.
     *
     * @return String
     */
    @RequestMapping(value = "cancelSearchNewParent.go", method = RequestMethod.GET)
    public String cancelSearchNewParent() {
      return REDIRECT + pageTrail.goToPreviousFlowUrl();
    }
}
