package gov.va.med.pharmacy.peps.presentation.common.controller.migration;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import gov.va.med.pharmacy.peps.presentation.common.controller.PepsController;

/**
 * This class contains operations shared by both the copay tier and the
 * formulary designator migration controllers.
 */
public abstract class AbstractMigrationController extends PepsController {
    /**
     * The class's logger.
     */
    private static final Logger LOG = 
        LogManager.getLogger(AbstractMigrationController.class);

    /**
     * Download generated migration EXCEL file. 
     * 
     * @param pResponse HttpServletResponse
     * @param migrationOutputFile File
     * @throws IOException IOException
     * @return true if no exceptions
     */
    protected boolean downloadExcel(HttpServletResponse pResponse, File migrationOutputFile) throws IOException {
          boolean successFlag = true;
          InputStream in = null;
          ServletOutputStream out = null;      
    
          try {
              in = new FileInputStream(migrationOutputFile);
              final int length = 256;   // 256 is arbitrary.
    
              pResponse.setContentType("application/octet-stream");
              pResponse.setHeader("Content-Disposition", "attachment;filename=" + migrationOutputFile.getName());
    
              out = pResponse.getOutputStream();
              byte[] outputByte = new byte[length];
              int size = 0;
    
              // Copy the input stream to the output stream using a buffer.
              while ((size = in.read(outputByte, 0, length)) != -1) {
                  out.write(outputByte, 0, size);
              }
              out.flush();
          } catch (IOException x) {        
              LOG.error("==>downloadExcelFile() " + x.getMessage());
              successFlag = false;
          } finally {
              if(in != null){
                  try {
                      in.close();
                  } catch (IOException e) {
                     LOG.error("Error closing the input stream");
                     LOG.error(e.getLocalizedMessage());
                  }
              }
              if (out!=null) {
                  try {
                      out.flush();
                      out.close();
                  } catch (IOException ioE) {
                      LOG.error("Error closing the output stream");
                      LOG.error(ioE.getLocalizedMessage());
                  }
              }
          }
          return successFlag;
      }
}
