package gov.va.med.pharmacy.peps.presentation.common.controller.rematch;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Set;

import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.common.vo.UnmatchRematchVo;

/**
 * A bean to hold the information needed for the rematch view.
 */
public class EditRematchBean {
  
  private MatchReason matchType;
  private List<UnmatchRematchVo> potentialMatches;
  private UnmatchRematchVo unmatchedProduct;
  private Long toAutoProductId;
  private Set<Long> toSuggestedProductIdSet;
  
  /**
   * Returns the matched automatic productId.
   * @return refer to description
   */
  public Long getToAutoProductId() {
    return toAutoProductId;
  }

  /**
   * Refer to {@link #getAutoProductId}.
   * @param autoProductId the id for the automatic product being matched to.
   */
  public void setToAutoProductId(Long autoProductId) {
      this.toAutoProductId = autoProductId;
  }

  /**
   * Returns the matched suggested product Id set.
   * @return refer to description.
   */
  public Set<Long> getToSuggestedProductIdSet() {
    return toSuggestedProductIdSet;
  }

  /**
   * Refer to {@link #getToSuggestedProductIdSet()}.
   * @param suggestedProductIdSet the set of ids for the suggested products being matched to.
   */
  public void setToSuggestedProductIdSet(Set<Long> suggestedProductIdSet) {
      this.toSuggestedProductIdSet = suggestedProductIdSet;
  }
  
  /**
   * Returns the list of product(s) to match to.
   * @return refer to description.
   */
  public Set<Long> getToProductIdSet() {
    if (toAutoProductId != null) {
      return Collections.singleton(toAutoProductId);
    }
    return toSuggestedProductIdSet;
  }
  
  /**
   * Return whether the given matched product(s) are automatic.
   * @return true if automatic, false otherwise.
   */
  public boolean isAuto() {
    if (toAutoProductId != null) {
      return true;
    }
    return false;
  }
  
  /**
   * Returns the rematch trigger. For example future inactivation.
   * @return refer to description.
   */
  public MatchReason getMatchType() {
    return matchType;
  }
  
  public void setMatchType(MatchReason matchType) {
    this.matchType = matchType;
   }
  
  /**
   * Returns true if the rematch trigger is a future inactivation, false otherwise.
   * @return refer to description.
   */
  public boolean getMatchTypeFuture() {
    return Objects.equals(matchType, MatchReason.FUTURE_INACTIVATION);
  }

  /**
   * Returns the list of potential matches for the unmatched product.
   * @return refer to description.
   */
  public List<UnmatchRematchVo> getPotentialMatches() {
    return potentialMatches;
  }
  
  public void setPotentialMatches(List<UnmatchRematchVo> productList) {
    this.potentialMatches = productList;
  }
  
  /**
   * Returns the product to be rematched.
   * @return refer to description.
   */
  public UnmatchRematchVo getUnmatchedProduct() {
    return unmatchedProduct;
  }
  
  public void setUnmatchedProduct(UnmatchRematchVo unmatchedProduct) {
    this.unmatchedProduct = unmatchedProduct;
  }
  
}
